@@ -2,7 +2,7 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  *
- *	Feature maintenance by  Matti Aarnio <mea@nic.funet.fi> 1991-1996
+ *	Feature maintenance by  Matti Aarnio <mea@nic.funet.fi> 1991-2000
  *
  */
 
@@ -13,6 +13,8 @@
 #include <sysexits.h>
 #include <ctype.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <pwd.h>
 #include "mail.h"
 #include "zmsignal.h"
@@ -22,7 +24,7 @@
 #include <sys/file.h>
 #include "zsyslog.h"
 
-#include "malloc.h"
+#include "zmalloc.h"
 #include "libz.h"
 #include "libc.h"
 
@@ -49,7 +51,7 @@ extern int  errno;
 extern int  optind;
 extern void doabort __((int));
 extern int  mail_priority;
-extern void check_and_print_to __((FILE *, const char*, const char*));
+extern void check_and_print_to __((FILE *, const char*, const char*, const char *));
 extern int  is_xtext_string __((const char *));
 
 const char *progname;
@@ -85,6 +87,48 @@ FILE	*mfp = NULL;
 	  cp = newcp;						\
 	}
 
+
+void usage()
+{
+  
+  fprintf(stderr, "Usage: %s [sendmail options] [recipient addresses]\n", progname);
+  fprintf(stderr, "  ZMailer's sendmail recognizes and implements following options:\n\
+     -B bodytype  -  Valid values: 8BITMIME, 7BIT\n\
+     -C conffile  -  specifies config file (meaningfull for -bt)\n\
+     -E           -  flag 'external' source\n\
+     -F 'full name'  sender's full name string\n\
+     -N notifyopt -  Notify option(s): NEVER or a set of: SUCCESS,DELAY,FAILURE\n\
+     -P priority# -  numeric priority for ZMailer router queue pre-selection\n\
+     -R returnopt -  Error report return option, either of: FULL, HDRS\n\
+     -U           -  Flag as 'user submission'\n\
+     -V envidstring - XTEXT encoded ENVID string\n\
+     -b?          -  operational mode flags\n\
+     -bd          -  starts smtpserver in daemon mode\n\
+     -bi          -  runs 'newaliases' command\n\
+     -bm          -  deliver mail; always :-)\n\
+     -bp          -  runs 'mailq' command\n\
+     -bs          -  speak smtp; runs smtpserver in interactive mode\n\
+     -bt          -  starts router in interactive test mode\n\
+     -e*             (ignored)\n\
+     -f fromaddr  -  sets envelope from address for the message\n\
+     -i           -  on inputs from tty this will ignore SMTP-like dot-EOF\n\
+     -m           -  send a copy of the message to the sender too (ignored)\n\
+     -o*          -  multiple options; those not listed cause error\n\
+     -oQ queuedir -  defines POSTOFFICE directory for message submission\n\
+     -ob*            (ignored)\n\
+     -od*            (ignored)\n\
+     -oe*            (ignored)\n\
+     -oi          -  alias of '-i' option\n\
+     -or*            (ignored)\n\
+     -p submitprotocol - value for 'with' label at 'Received:' header\n\
+     -q*          -  queue processing commands (ignored)\n\
+     -r fromaddr  -  (alternate for -f)\n\
+     -t           -  scan message rfc822 headers for recipient addresses\n\
+     -v           -  verbose trace of processing\n\
+");
+}
+
+
 extern int main __((int argc, const char *argv[]));
 int
 main(argc, argv)
@@ -93,7 +137,7 @@ main(argc, argv)
 {
 	int	c, errflg, n, pid, bpflag, uid, truid, external, verbose;
 	int	speaksmtp, daemon_flg, interactive, aliases, printq, dotiseof;
-	int	binary, vfd;
+	int	rfc822recipients, vfd;
 	char	*ebp, *s, *cp, errbuf[BUFSIZ];
 	const char *from, *mailbin, *mailshare, *fullname;
 	const char *mav[10];
@@ -111,6 +155,8 @@ main(argc, argv)
 	char	*returnopt = NULL;
 	char	*envidstr = NULL;
 	const char * newcp = NULL;
+	int	save_from = 0;
+	int	outcount = 0;
 
 	cp = strrchr(argv[0], '/');
 	if (cp != NULL)
@@ -120,17 +166,16 @@ main(argc, argv)
 	umask(022);
 
 	mailpriority = getenv("MAILPRIORITY");
-	if (mailpriority)
-	  mail_priority = atoi(mailpriority);
 
 	/* Pick up the sender's idea about C-chartype.. */
 	LC_ctype = getenv("LC_CTYPE");
 
-	printq = strcmp(progname, MAILQ) == 0;
-	aliases = strcmp(progname, NEWALIASES) == 0;
+	printq  = STREQ(progname, MAILQ);
+	aliases = STREQ(progname, NEWALIASES);
+
 	from = fullname = NULL;
 	speaksmtp = daemon_flg = interactive = 0;
-	bpflag = external = verbose = binary = 0;
+	bpflag = external = verbose = rfc822recipients = 0;
 	dotiseof = 1;	/* tested below to see if changed */
 	ebp = errbuf;
 	*ebp = '\0';
@@ -148,6 +193,12 @@ main(argc, argv)
 			/* Sendmail 8.7 compability:
 			   -B8BITMIME */
 			bodytype = optarg;
+			if (strcasecmp(bodytype,"8BITMIME") != 0 &&
+			    strcasecmp(bodytype,"7BIT") != 0 &&
+			    strcasecmp(bodytype,"BINARYMIME") != 0) {
+			  fprintf(stderr,"sendmail: unrecognized -B option parameter value: '%s'\n",bodytype);
+			  exit(EX_USAGE);
+			}
 			break;
 		case 'J': break; /* Sony NEWS OS  JIS-conversion option,
 				    ignore */
@@ -162,7 +213,7 @@ main(argc, argv)
 			fullname = optarg;
 			break;
 		case 't':	/* scan header for recipient addresses */
-			binary = 1;
+			rfc822recipients = 1;
 			break;
 		case 'b':
 			switch (*optarg) {
@@ -226,9 +277,13 @@ main(argc, argv)
 			case 'i':	/* ignore dots */
 				dotiseof = 0;
 				break;
+			case 'r':
+				break;
 			default:
+#if 0
 				sprintf(ebp, " -o%s", optarg);
 				ebp += strlen(ebp);
+#endif
 				break;
 			}
 			break;
@@ -248,14 +303,17 @@ main(argc, argv)
 			{
 			  s = optarg;
 			  while (*s) {
-			    if (cistrncmp(s,"never",5)==0)
+			    if (CISTREQN(s,"NEVER",5))
 			      s += 5;
-			    else if (cistrncmp(s,"success",7)==0)
+			    else if (CISTREQN(s,"SUCCESS",7))
 			      s += 7;
-			    else if (cistrncmp(s,"failure",7)==0)
+			    else if (CISTREQN(s,"FAILURE",7))
 			      s += 7;
-			    else if (cistrncmp(s,"delay",5)==0)
+			    else if (CISTREQN(s,"DELAY",5))
 			      s += 5;
+			    else if (CISTREQN(s,"TRACE",5))
+			      s += 5; /* This is NOT even RFC 2852 derived
+					 thing, although helps to debug it.. */
 			    if (*s == ',') {
 			      ++s;
 			      continue;
@@ -269,8 +327,8 @@ main(argc, argv)
 			break;
 		case 'R':
 			returnopt = optarg;
-			if (cistrcmp(returnopt,"full") != 0 &&
-			    cistrcmp(returnopt,"hdrs") != 0) {
+			if (!CISTREQ(returnopt,"FULL") &&
+			    !CISTREQ(returnopt,"HDRS")) {
 			  fprintf(stderr,"sendmail: illegal -R -option parameter: '%s'\n",optarg);
 			  exit(EX_USAGE);
 			}
@@ -282,14 +340,21 @@ main(argc, argv)
 			verbose = 1;
 			break;
 		case 's':	/* save From_ lines */
-			sprintf(ebp, " -%c", c);
-			ebp += strlen(ebp);
+			save_from = 1;
 			break;
 		case 'p':
 			submitprotocol = optarg;
+			for (;*optarg;++optarg) {
+			  int c = (*optarg) & 0xFF;
+			  if ('0' <= c && c <= '9') continue;
+			  if ('A' <= c && c <= 'Z') continue;
+			  if ('z' <= c && c <= 'z') continue;
+			  fprintf(stderr,"sendmail: only alphanumeric characters accepted for -p option parameter: '%s'\n",submitprotocol);
+			  exit(EX_USAGE);
+			}
 			break;
 		case 'P':
-			mail_priority = atoi(optarg);
+			mailpriority = optarg;
 			break;
 		case 'V':
 			envidstr = optarg;
@@ -306,6 +371,23 @@ main(argc, argv)
 		}
 	}
 
+	mail_priority = _MAILPRIO_NORMAL;
+	if (mailpriority) {
+	  mail_priority = atoi(mailpriority);
+	  if (mail_priority < 0) {
+	    /* Some word ?? */
+	    if (CISTREQ(mailpriority,"high"))
+	      mail_priority = _MAILPRIO_HIGH;
+	    else if (CISTREQ(mailpriority,"NORMAL"))
+	      mail_priority = _MAILPRIO_NORMAL;
+	    else if (CISTREQ(mailpriority,"BULK"))
+	      mail_priority = _MAILPRIO_BULK;
+	    else if (CISTREQ(mailpriority,"JUNK"))
+	      mail_priority = _MAILPRIO_JUNK;
+	    else
+	      mail_priority = _MAILPRIO_NORMAL;
+	  }
+	}
 	/* Make sure the submission priority is >= 0  */
 	if (mail_priority < 0) mail_priority = 0;
 
@@ -318,8 +400,7 @@ otherprog:
 				zmailer);
 	}
 	if (errflg) {
-		fprintf(stderr, "Usage: %s [sendmail options]\n",
-				       progname);
+		usage();
 		exit(EX_USAGE);
 	}
 	n = 0;
@@ -359,15 +440,12 @@ otherprog:
 			exit(EX_UNAVAILABLE);
 		}
 		if ((pid = fork()) > 0) {
-			mav[1] = "-dk";
-			execv(ROUTER, (char*const*)mav);
-			perror(ROUTER);
-			exit(EX_UNAVAILABLE);
-		} else if (pid == 0) {
-			mav[0] = "scheduler";
-			mav[1] = "-d";
-			execv(SCHEDULER, (char*const*)mav);
-			perror(SCHEDULER);
+			exit(0);
+		} else if (pid == 0) { /* Child */
+			mav[0] = "zmailer";
+			mav[1] = NULL;
+			execv("zmailer", (char*const*)mav);
+			perror(zmailer);
 			exit(EX_UNAVAILABLE);
 		}
 		perror("fork");
@@ -457,6 +535,7 @@ otherprog:
 
 		vfd = -1;
 		if (verbose) {
+		  int old_umask;
 		  if (postoffice == NULL) {
 		    postoffice = getzenv("POSTOFFICE");
 		    if (postoffice == NULL)
@@ -466,25 +545,33 @@ otherprog:
 				     +strlen(PUBLICDIR)+20);
 		  sprintf(verbfile, "%s/%s/v_XXXXXX",
 			  postoffice, PUBLICDIR);
+		  old_umask = umask(0077);
+#ifdef HAVE_MKSTEMP
+		  vfd = mkstemp(verbfile);
+		  if (*verbfile == '\0' || vfd < 0)
+#else
 		  mktemp(verbfile);
 		  if (*verbfile == '\0' ||
-		      (vfd = open(verbfile, O_CREAT|O_RDWR, 0600)) < 0) {
-		    fprintf(stderr,
-			    "%s: cannot create verbose log file in %s/%s\n",
-			    zmailer, postoffice, PUBLICDIR);
-		    verbfile = NULL;
-		  } else {
-		    /*
-		     * We need to make it a relative pathname
-		     * in case the router/scheduler's idea of
-		     * root directory is different than ours.
-		     */
-		    cp = strrchr(verbfile, '/');
-		    while (--cp > verbfile)
-		      if (*cp == '/')
-			break;
-		    fprintf(mfp, "verbose \"../%s\"\n", cp+1);
-		  }
+		      (vfd = open(verbfile, O_CREAT|O_RDWR, 0600)) < 0)
+#endif
+		    {
+		      fprintf(stderr,
+			      "%s: cannot create verbose log file in %s/%s\n",
+			      zmailer, postoffice, PUBLICDIR);
+		      verbfile = NULL;
+		    } else {
+		      /*
+		       * We need to make it a relative pathname
+		       * in case the router/scheduler's idea of
+		       * root directory is different than ours.
+		       */
+		      cp = strrchr(verbfile, '/');
+		      while (--cp > verbfile)
+			if (*cp == '/')
+			  break;
+		      fprintf(mfp, "verbose \"../%s\"\n", cp+1);
+		    }
+		  umask(old_umask);
 		}
 		if (external)
 		  fprintf(mfp, "external\n");
@@ -492,12 +579,13 @@ otherprog:
 		  fprintf(mfp, "bodytype %s\n", bodytype);
 		if (fullname != NULL && *fullname != '\0')
 		  /* maybe we should put it in the environment? */
-
 		  fprintf(mfp, "fullname %s\n", fullname);
 
 		RFC821_822QUOTE(newcp,from);
 
-		if (from != NULL && *from != '\0')
+		if (from != NULL && STREQ(from,"<>"))
+		  fprintf(mfp, "channel error\n");
+		else if (from != NULL && *from != '\0')
 		  fprintf(mfp, "from %s\n", from);
 
 		if (newcp) free((void*)newcp); newcp = NULL;
@@ -509,8 +597,9 @@ otherprog:
 		if (submitprotocol)
 		  fprintf(mfp, "with %s\n", submitprotocol);
 
-		for (; optind < argc; ++optind)
-		  check_and_print_to(mfp,argv[optind],notificationopt);
+		if (!rfc822recipients)
+		  for (; optind < argc; ++optind)
+		    check_and_print_to(mfp,argv[optind],notificationopt,from);
 
 		fprintf(mfp,"env-end\n");
 
@@ -529,10 +618,13 @@ otherprog:
 		 * which, though it may be unsatisfactory, is good for security.
 		 */
 		
-		if (binary) {
+		if (rfc822recipients) {
+		  /* We don't mess with headers, nor the envelope at
+		     this phase. */
 		  while ((n = read(0, buf, sizeof buf)) > 0) {
 		    if (fwrite(buf, sizeof buf[0], n, mfp) != n)
 		      break;
+		    outcount += n;
 		  }
 		} else {
 		  /* cmd line can only set dotiseof to 0.. improvement? */
@@ -563,7 +655,7 @@ otherprog:
 			s[0] = '\n';
 			s[1] = 0;
 		      }
-		      if (strncmp(buf,"From ",5)==0) {
+		      if (!save_from && STREQN(buf,"From ",5)) {
 			/* [mea@utu.fi] I vote for
 			   removing this line if next
 			   is a RFC-header */
@@ -572,7 +664,7 @@ otherprog:
 			strcpy(buf2,buf);
 			continue;
 		      }
-		      if (strncmp(buf,">From ",6)==0) {
+		      if (!save_from && STREQN(buf,">From ",6)) {
 			/* [mea@utu.fi] I vote for
 			   removing this line if next
 			   is a RFC-header */
@@ -604,6 +696,7 @@ otherprog:
 		    if (dotiseof && *s == '.' && *++s == '\n')
 		      break;
 		    fputs(buf, mfp);
+		    ++outcount;
 		  }
 		}
 
@@ -617,6 +710,12 @@ otherprog:
 		  if (vfd >= 0)
 		    unlink(verbfile);
 		  exit(EX_IOERR);
+		} else if (outcount == 0) {
+		  /* No input ?? Ignore silently */
+		  mail_abort(mfp);
+		  if (vfd >= 0)
+		    unlink(verbfile);
+		  exit(EX_OK);
 		} else if (mail_close(mfp) == EOF) {
 		  fprintf(stderr, "%s: message not submitted!\n",
 			  zmailer);
@@ -635,7 +734,7 @@ otherprog:
 		  while (vfp != NULL) {
 		    while (fgets(buf, sizeof buf, vfp) != NULL) {
 		      fprintf(stderr, "%s", buf);
-		      if (strncmp(buf, "scheduler done", 14) == 0)
+		      if (STREQN(buf, "scheduler done", 14))
 			sleeplimit = 20; /* 20 seconds to death */
 		    }
 		    clearerr(vfp);
@@ -697,11 +796,11 @@ int dummy;
 
 /* haa@cs.hut.fi:  Sometimes address components contain pure junk.. */
 void
-check_and_print_to(mfp, addr, notify)
+check_and_print_to(mfp, addr, notify, from)
 	FILE *mfp;
-	const char *addr, *notify;
+	const char *addr, *notify, *from;
 {
-	const char *copy = NULL, *printme = addr, *from;
+	const char *copy = NULL, *printme = addr, *frm;
 	const char *s, *newcp = NULL;
 	char *to;
 
@@ -715,8 +814,8 @@ check_and_print_to(mfp, addr, notify)
 	  if (to == NULL) return;	  /* should never happen... */
 
 	  copy = to;
-	  for (from = addr; *from; ++from)
-	    if (*from != '\n') *to++ = *from;
+	  for (frm = addr; *frm; ++frm)
+	    if (*frm != '\n') *to++ = *frm;
 	  *to = 0;
 
 	  printme = copy;
@@ -735,6 +834,31 @@ check_and_print_to(mfp, addr, notify)
 	    fprintf(mfp,"+%02X",c);
 	  ++s;
 	}
+
+	fprintf(mfp, " INRCPT=rfc822;");
+	s = printme;
+	while (*s) {
+	  u_char c = *s;
+	  if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	    putc(c,mfp);
+	  else
+	    fprintf(mfp,"+%02X",c);
+	  ++s;
+	}
+
+	if (from) {
+	  fprintf(mfp, " INFROM=rfc822;");
+	  s = from;
+	  while (*s) {
+	    u_char c = *s;
+	    if ('!' <= c && c <= '~' && c != '+' && c != '=')
+	      putc(c,mfp);
+	    else
+	      fprintf(mfp,"+%02X",c);
+	    ++s;
+	  }
+	}
+
 	if (notify)
 	  fprintf(mfp," NOTIFY=%s", notify);
 	putc('\n',mfp);
