@@ -11,6 +11,13 @@
 # endif
 #endif
 
+#ifndef CISTREQ
+#define  CISTREQ(x,y)	 (cistrcmp ((const char*)(x), (const char*)(y)  )==0)
+#define  CISTREQN(x,y,n) (cistrncmp((const char*)(x), (const char*)(y),n)==0)
+#define  STREQ(x,y)      (strcmp   ((const char*)(x), (const char*)(y)  )==0)
+#define  STREQN(x,y,n)   (strncmp  ((const char*)(x), (const char*)(y),n)==0)
+#endif
+
 /* allocate.c */
 #ifdef MEMTYPES
 extern memtypes	 stickymem;
@@ -62,7 +69,7 @@ extern int estat  __((const char *path, struct stat *stbuf));
 #endif
 extern off_t elseek __((int fd, off_t pos, int action));
 extern int elink    __((const char *file1, const char *file2));
-extern int eunlink  __((const char *file));
+extern int eunlink  __((const char *file, const char *tag));
 extern int eclose   __((int fd));
 extern int echdir   __((const char *file));
 extern int emkdir   __((const char *file, int mode));
