@@ -33,6 +33,7 @@ struct taddress {
 #define _DSN_NOTIFY_FAILURE	0x002
 #define _DSN_NOTIFY_DELAY	0x004
 #define _DSN_NOTIFY_NEVER	0x008
+#define _DSN_NOTIFY_TRACE	0x010
 
 #define _DSN__DIAGDELAYMODE	 0x800 /* Internal magic for MAILBOX/SIEVE */
 #define _DSN__TEMPFAIL_NO_UNLOCK 0x400 /* Internal magic for SMTP */
@@ -55,8 +56,14 @@ struct rcpt {
 	struct taddress	*addr;		/* addr.link is the sender address */
 	const char	*orcpt;		/*  DSN  ORCPT=  string */
 	const char	*inrcpt;	/* "DSN" INRCPT= string */
+	const char	*infrom;	/* "DSN" INFROM= string */
 	const char	*notify;	/*  DSN  NOTIFY= flags  */
 	int		notifyflgs;
+	time_t		deliverby;	/*  RFC 2852  DELIVERBY */
+	int		deliverbyflgs;
+#define _DELIVERBY_N 1
+#define _DELIVERBY_R 2
+#define _DELIVERBY_T 4
 	char		***newmsgheader; /* message header line pointer ptr
 					   that points to an address of
 					      ctldesc->msgheaders[index]
@@ -172,35 +179,6 @@ extern int	       emptyline __(( char *line, int size ));
 
 extern int zmalloc_failure;
 
-#if 0 /* actually better to include "libz.h" for this */
-/* esyslib.c: */
-extern int cistrcmp  __((const char *s1, const char *s2));
-extern int cistrncmp __((const char *s1, const char *s2, int n));
-extern int efstat  __((int fd, struct stat *stbuf));
-extern int emkdir  __((const char *dirpath, int mode));
-extern int eopen   __((const char *s, int f, int m));
-extern int epipe   __((int fdarr[2]));
-extern int eread   __((int fd, char *buf, int len));
-extern int estat   __((const char *filename, struct stat *stbuf));
-extern int eunlink __((const char *filename));
-extern int errno;
-extern int optind;
-extern int embytes;
-extern int emcalls;
-extern unsigned emsleeptime;
-extern void * emalloc  __((size_t len));
-extern void * erealloc __((void * buf, size_t len));
-#ifdef S_IWUSR /* Defined on <sys/stat.h> */
-extern int efstat __((int fd, struct stat *stbuf));
-#endif
-extern long elseek __((int fd, off_t pos, int action));
-extern int elink __((const char *file1, const char *file2));
-extern int eclose __((int fd));
-extern int echdir __((const char *path));
-extern int ermdir __((const char *path));
-extern int erename __((char *from, char *to));
-#endif
-
 /* lockaddr.c: */
 extern int lockaddr __((int fd, char *map, int offset, int was, int new, const char *file, const char *host, const int mypid));
 
@@ -255,11 +233,6 @@ extern char *skip821address __((const char *s));
 extern void tatimestr __((char *buf, int dt));
 extern void tasyslog __((struct rcpt *rp, int xdelay, const char *wtthost, const char *wttip, const char *stats, const char *msg));
 
-#ifndef CISTREQ
-#define  CISTREQ(x, y)      (cistrcmp((const char *)(x), (const char *)(y)) == 0)
-#define  CISTREQN(x, y, n)  (cistrncmp((const char *)(x), (const char *)(y), n) == 0)
-#endif
-
 extern int getmyuucename __((char *, int));
 
 /* nonblocking.c */
