@@ -4,7 +4,7 @@
  */
 /*
  *	A lot of changes all around over the years by Matti Aarnio
- *	<mea@nic.funet.fi> (and elsewere), copyright 1992-1997
+ *	<mea@nic.funet.fi> (and elsewere), copyright 1992-2000
  */
 
 #include "mailer.h"
@@ -12,7 +12,7 @@
 #include "libsh.h"
 
 static const char *Copyright = "Copyright 1992 Rayan S. Zachariassen\n\
-Copyright 1992-1997 Matti Aarnio";
+Copyright 1992-2000 Matti Aarnio";
 
 void
 prversion(prgname)
@@ -20,4 +20,6 @@ prversion(prgname)
 {
 	fprintf(stderr, "ZMailer %s (%s)\n  %s:%s\n%s\n", prgname,
 		Version, CC_user, CC_pwd, Copyright);
+	fprintf(stderr, "Configured with command: '%s'\n",
+		CONFIGURE_CMD);
 }
