@@ -9,65 +9,22 @@
  */
 
 #include "hostenv.h"
+
 #include <stdio.h>
 #include <sys/types.h>
 
 #include "splay.h"
 #include "zmalloc.h"
-#include "libz.h"
 
 #ifdef	symbol
 #undef	symbol
 #endif	/* symbol */
 
 extern long pjwhash32n __((const void *, int));
+extern long crc32n     __((const void *, int));
 
 
-/* crc table and hash algorithm from pathalias */
-/*
- * fold a string into a long int.  31 bit crc (from andrew appel).
- * the crc table is computed at run time by crcinit() -- we could
- * precompute, but it takes 1 clock tick on a 750.
- *
- * This fast table calculation works only if POLY is a prime polynomial
- * in the field of integers modulo 2.  Since the coefficients of a
- * 32-bit polynomail won't fit in a 32-bit word, the high-order bit is
- * implicit.  IT MUST ALSO BE THE CASE that the coefficients of orders
- * 31 down to 25 are zero.  Happily, we have candidates, from
- * E. J.  Watson, "Primitive Polynomials (Mod 2)", Math. Comp. 16 (1962):
- *      x^32 + x^7 + x^5 + x^3 + x^2 + x^1 + x^0
- *      x^31 + x^3 + x^0
- *
- * We reverse the bits to get:
- *      111101010000000000000000000000001 but drop the last 1
- *         f   5   0   0   0   0   0   0
- *      010010000000000000000000000000001 ditto, for 31-bit crc
- *         4   8   0   0   0   0   0   0
- */
-
-#define POLY32 0xf5000000       /* 32-bit polynomial */
-#define POLY31 0x48000000       /* 31-bit polynomial */
-#define POLY POLY31     /* use 31-bit to avoid sign problems */
-
-static long CrcTable[128];
-static int crcinit_done = 0;
-
-static void crcinit __((void));
-static void
-crcinit()
-{       
-	register int i,j;
-	register long sum;
-
-	for (i = 0; i < 128; i++) {
-		sum = 0;
-		for (j = 7-1; j >= 0; --j)
-			if (i & (1 << j))
-				sum ^= POLY >> j;
-		CrcTable[i] = sum;
-	}
-	crcinit_done = 1;
-}
+#define CRCorNOT 1 /* 0: pjwhash32n(), !0: crc32n() */
 
 struct syment {
 	struct syment *next;
@@ -102,23 +59,16 @@ symbol_lookup_db_mem_(s, slen, spt, usecrc)
 	struct sptree *spt;
 	const int usecrc;
 {
-	register const char *ucp;
 	register spkey_t key;
 	register struct syment *se, *pe;
 	struct spblk *spl;
-	int i = slen;
 
 	if (s == NULL)
 		return 0;
 
-	if (usecrc) {
-	  if (!crcinit_done)
-	    crcinit();
-	  /* Input string is to be CRCed to form a new key-id */
-	  key = 0;
-	  for (ucp = s; i > 0; ++ucp, --i)
-	    key = (key >> 7) ^ CrcTable[(key ^ *ucp) & 0x7f];
-	} else
+	if (usecrc)
+	  key = crc32n(s, slen);
+	else
 	  key = pjwhash32n(s, slen);
 
 	/* Ok, time for the hard work.  Lets see if we have this key
@@ -150,7 +100,7 @@ symbol_lookup_db_mem(s, slen, spt)
 	const int slen;
 	struct sptree *spt;
 {
-  return symbol_lookup_db_mem_(s, slen, spt, 0);
+  return symbol_lookup_db_mem_(s, slen, spt, CRCorNOT);
 }
 
 
@@ -161,23 +111,16 @@ symbol_db_mem_(s, slen, spt, usecrc)
 	int slen, usecrc;
 	struct sptree *spt;
 {
-	register const char *ucp;
 	register spkey_t key;
 	register struct syment *se, *pe;
 	struct spblk *spl;
-	int i = slen;
 
 	if (s == NULL)
 		return 0;
 
-	if (usecrc) {
-	  if (!crcinit_done)
-	    crcinit();
-	  /* Input string is to be CRCed to form a new key-id */
-	  key = 0;
-	  for (ucp = s; i > 0; ++ucp, --i)
-	    key = (key >> 7) ^ CrcTable[(key ^ *ucp) & 0x7f];
-	} else
+	if (usecrc)
+	  key = crc32n(s,slen);
+	else
 	  key = pjwhash32n(s,slen);
 
 	/* Ok, time for the hard work.  Lets see if we have this key
@@ -219,7 +162,7 @@ symbol_db_mem(s, slen, spt)
 	int slen;
 	struct sptree *spt;
 {
-  return symbol_db_mem_(s, slen, spt, 0);
+  return symbol_db_mem_(s, slen, spt, CRCorNOT);
 }
 
 /*
@@ -261,23 +204,18 @@ symbol_free_db_mem_(s, slen, spt, usecrc)
 	int slen, usecrc;
 	struct sptree *spt;
 {
-	register const char *ucp;
 	register spkey_t key;
 	register struct syment *se, *pe;
 	struct spblk *spl;
-	int i = slen;
 
 	if (s == NULL || spt == NULL)
 		return;
 
-	if (usecrc) {
-	  /* Input string is to be CRCed to form a new key-id */
-	  key = 0;
-	  for (ucp = s; i >= 0; ++ucp, --i)
-	    key = (key >> 7) ^ CrcTable[(key ^ *ucp) & 0x7f];
-	} else {
+	if (usecrc)
+	  key = crc32n(s, slen);
+	else
 	  key = pjwhash32n(s, slen);
-	}
+
 
 	/* Ok, time for the hard work.  Lets see if we have this key
 	   in the symtab splay tree (we can't use cache here!) */
@@ -314,7 +252,7 @@ void symbol_free_db_mem(s, slen, spt)
 	int slen;
 	struct sptree *spt;
 {
-	symbol_free_db_mem_(s, slen, spt, 0);
+	symbol_free_db_mem_(s, slen, spt, CRCorNOT);
 }
 
 void
@@ -380,4 +318,3 @@ symbol_db(s, spt)
 		return 0;
 	return symbol_db_mem(s, strlen(s), spt);
 }
-
