@@ -54,10 +54,12 @@ static conscell *sh_cdr		CSARGS2;
 static conscell *sh_list	CSARGS2;
 static conscell *sh_grind	CSARGS2;
 static conscell *sh_elements	CSARGS2;
-static conscell *sh_setf	CSARGS2;
 static conscell *sh_get		CSARGS2;
 static conscell *sh_length	CSARGS2;
 static conscell *sh_last	CSARGS2;
+static conscell *sh_lappend     CSARGS2;
+static conscell *sh_lreplace    CSARGS2;
+extern conscell *sh_glob	CSARGS2; /* expand.c */
 
 #define CSARGV2 __((int argc, const char *argv[]))
 
@@ -91,7 +93,6 @@ struct shCmd builtins[] = {
 {	"list",		NULL,	sh_list,	NULL,	SH_ARGV		},
 {	"grind",	NULL,	sh_grind,	NULL,	SH_ARGV		},
 {	"elements",	NULL,	sh_elements,	NULL,	SH_ARGV		},
-{	"setf",		NULL,	sh_setf,	NULL,	SH_ARGV		},
 {	"get",		NULL,	sh_get,		NULL,	SH_ARGV		},
 {	"length",	NULL,	sh_length,	NULL,	SH_ARGV		},
 {	"[",		sh_test,	NULL,	NULL,	0		},
@@ -122,26 +123,25 @@ struct shCmd builtins[] = {
 {	"sleep",	sh_sleep,	NULL,	NULL,	0		},
 {	"true",		sh_true,	NULL,	NULL,	0		},
 {	"false",	sh_false,	NULL,	NULL,	0		},
+{	"lappend",	NULL,	sh_lappend,	NULL,	SH_ARGV		},
+{	"lreplace",	NULL,	sh_lreplace,	NULL,	SH_ARGV		},
+{	"glob",		NULL,	sh_glob,	NULL,	SH_ARGV		},
 {	NULL,		NULL,		NULL,	NULL,	0		},
 };
 
 conscell *
 sh_car(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
-	conscell *tmp;
-
 	il = cdar(avl);
 	if (il == NULL)
 		return NULL;
-	if (STRING(il))
-		return conststring(il->string);
-	/* setf preparation */
+	if (STRING(il)) {
+		il = copycell(il);
+		cdr(il) = NULL;
+		return il;
+	}
 	if (car(il) == NULL) {
-		if (il->prev) {
-			il->prev = cdr(il->prev);
-			il->pflags = 0;
-		}
 		return il;
 	}
 	car(il) = copycell(car(il));	/* don't modify malloc'ed memory! */
@@ -151,48 +151,30 @@ sh_car(avl, il)
 
 static conscell *
 sh_cdr(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
 		return NULL;
-	/* setf preparation */
-	if (cdar(il)) {
-		il->prev = cdar(il)->prev;
-	} else if (car(il)->prev) {
-		if (car(il)->pflags)
-			il->prev = cdr(car(il)->prev);
-		else
-			il->prev = car(car(il)->prev);
-	}
-	il->pflags = 3;
 	car(il) = cdar(il);
 	return il;
 }
 
 static conscell *
 sh_last(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
 	il = cdar(avl);
 	if (il == NULL || STRING(il) || car(il) == NULL)
 		return NULL;
-	/* setf preparation */
 	while (cdar(il) != NULL)
 		car(il) = cdar(il);
-	if (car(il)->prev) {
-		/* if (car(il)->pflags)*/
-			il->prev = car(il)->prev;
-		/*else
-			il->prev = car(car(il)->prev);*/
-	}
-	il->pflags = 3;
 	return il;
 }
 
 static conscell *
 sh_list(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
 	car(avl) = cdar(avl);
 	return avl;
@@ -200,104 +182,213 @@ sh_list(avl, il)
 
 static conscell *
 sh_grind(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
 	return cdar(avl);
 }
 
 static conscell *
 sh_elements(avl, il)
-	conscell *avl, *il;
+	conscell *avl, *il; /* Input conscells are gc-protected! */
 {
 	conscell *p;
 
 	if ((p = cdar(avl)) == NULL || !LIST(p))
 		return p;
-	for (il = cadar(avl); il != NULL; il = cdr(il))
-		il->flags |= ELEMENT;
-	return cadar(avl);
+	il = cadar(avl);
+
+	il = s_copy_chain(il); /* Creates new cells, but this is
+				  also last call to the creator here. */
+
+	for (p = il; p != NULL; p = cdr(p))
+		p->flags |= ELEMENT;
+	return il;
 }
 
-static conscell *
-sh_setf(avl, il)
-	conscell *avl, *il;
-{
-	conscell *place, *value, *tmp = NULL;
-	memtypes oval = stickymem;
 
-	/*
-	 * The `place' argument to setf should NOT be malloc'ed stuff,
-	 * minor tweaks may be necessary in car/cdr/get (certainly the latter)
-	 * to ensure this.
-	 */
-	if ((place = cdar(avl)) == NULL) {
-		fprintf(stderr, "Usage: %s variable-reference [new-value]\n",
+/*
+ *  call: lappend varname $value
+ *  The varname is looked up, and appended with supplied value;
+ *  approach 
+ *
+ */
+
+static conscell *
+sh_lappend(avl, il)
+	conscell *avl, *il; /* Input conscells are gc-protected! */
+{
+	conscell *key, *d, *tmp, *data;
+	memtypes omem = stickymem;
+
+	key = cdar(avl);
+	if (key == NULL 
+	    || !STRING(key)) {
+		fprintf(stderr, "Usage: %s variable-name $moredata\n",
 				car(avl)->string);
 		return NULL;
 	}
-	value = cddar(avl);
+	d = v_find(key->string); /* no new objects allocated */
+	if (!d) return NULL;
 
-	if (place->prev == NULL)
-		return value;
+	d = cdr(d); /* This is variable content pointing object */
 
 	stickymem = MEM_MALLOC;
 
-	value = s_copy_tree(value);
-	if (place->pflags) {
-		if (value != NULL && (place->pflags & 02)) {
-			if (LIST(value)) {
-				tmp = car(value);
-				free((char *)value);
-				value = tmp;
-			}
-			place->pflags &= ~02;
-		}
-		if (place->pflags & 04) {
-			if ((tmp = cdr(place->prev))) {
-				tmp = cdr(tmp);
-				if (value != NULL) {
-					cddr(place->prev) = NULL;
-					s_free_tree(cdr(place->prev));
-				}
-			}
-		} else {
-			if (cdr(place->prev) != NULL)
-				s_free_tree(cdr(place->prev));
-			cdr(place->prev) = NULL;
-		}
-		if (value != NULL) {
-			cdr(place->prev) = value;
-			s_set_prev(place->prev, value);
-			value->pflags = 1;
-			if (place->pflags & 04) {
-				value = s_last(value);
-				if (cdr(value))
-					s_free_tree(cdr(value));	/* XX */
-				cdr(value) = tmp;
-			}
-		}
+	tmp = cdr(key); /* data is the next elt after the variable name */
+	if (LIST(tmp))  /* If it is a LIST object, descend into it */
+	  tmp = car(tmp);
+	data = s_copy_chain(tmp); /* The only cell-allocator in here */
+
+	if (car(d) != NULL) {
+	  d = car(d);
+
+	  while (cdr(d)) d = cdr(d); /* Scan to the end of the list */
+	
+	  cdr(d) = data;
 	} else {
-		if (value != NULL && car(place->prev) != NULL) {
-			if (cdar(place->prev))
-				cdr(s_last(value)) = cdar(place->prev);
-			cdar(place->prev) = NULL; /* for free() below */
-		}
-		s_free_tree(car(place->prev));
-		car(place->prev) = value;
-		s_set_prev(place->prev, value);
+	  car(d) = data; /* the first entry into the list */
 	}
-	stickymem = oval;
-#ifdef	MAILER
-	v_touched();
-#endif	/* MAILER */
-	return cddar(avl);
+
+	stickymem = omem;
+	return NULL; /* Be quiet, don't force the caller
+			to store the result into heap just
+		        for latter discard... */
 }
 
+/*
+ *  call: lreplace list_var_name fieldidx $new_value
+ *  The varname is looked up, and indicated element of it
+ *  is replaced with a new value.
+ *
+ *  This processes 1) linear lists where index is decimal value,
+ *  and 2) key/value pair lists where 'index' is key name
+ *
+ *  If you are referring beyond end of the list (or to a key
+ *  which does not exist), the value (or key + value) is (are)
+ *  appended to the variable.
+ */
+
+static conscell *
+sh_lreplace(avl, il)
+	conscell *avl, *il;
+{
+	conscell *key, *d, *tmp, *data, **dp;
+	memtypes omem = stickymem;
+	int fieldidx, fieldnamelen;
+	char *fieldname;
+
+	GCVARS4;
+
+	const char *lreplace_usage =
+	  "Usage: %s variable-name fieldidx $new_value\n\
+    where 'fieldidx' can be 1) numeric for index in a list (0..),\n\
+    or 2) key/value list's key name\n";
+
+	key = cdar(avl);
+	if (key == NULL 
+	    || !STRING(key)) {
+		fprintf(stderr, lreplace_usage, car(avl)->string);
+		return NULL;
+	}
+	d = v_find(key->string);
+	if (!d) {
+	  fprintf(stderr, lreplace_usage, car(avl)->string);
+	  return NULL;
+	}
+
+	tmp = cdr(key);  /* Numeric value for field index */
+	if (!tmp || !STRING(tmp)) {
+		fprintf(stderr, lreplace_usage, car(avl)->string);
+		return NULL;
+	}
+	if ('0' <= tmp->string[0] && tmp->string[0] <= '9') {
+	  fieldidx = -1;
+	  if (sscanf(tmp->string,"%i",&fieldidx) != 1 ||
+	      fieldidx < 0 || fieldidx > 99) {
+	    fprintf(stderr, lreplace_usage, car(avl)->string);
+	    return NULL;
+	  }
+	  fieldname = NULL;
+	  fieldnamelen = 0;
+	} else {
+	  fieldname    = tmp->string;
+	  fieldnamelen = tmp->slen;
+	}
+
+	stickymem = MEM_MALLOC;
+
+	tmp = cdr(tmp); /* data is the next elt after the field index */
+#if 0
+	if (LIST(tmp))  /* If it is a LIST object, descend into it */
+	  tmp = car(tmp);
+#endif
+
+	data = NULL;
+	GCPRO4(data, tmp, d, key);
+
+	data = s_copy_chain(tmp);
+
+	dp = &car(cdr(d)); /* This is variable pointing object */
+
+	/* Must make copy of this chain here! */
+	*dp = s_copy_chain(*dp);
+
+	d = *dp;
+	if (fieldname) {
+	  while (d != NULL) {
+	    /* Ok, this is key/value pairs in a linear list */
+	    if (STRING(d) && fieldnamelen == d->slen &&
+		memcmp(d->string,fieldname,fieldnamelen)==0) {
+	      /* Move pointers to the value */
+	      dp = &cdr(d);
+	      d = *dp;
+	      break;
+	    }
+	    dp = &cdr(d);
+	    d = *dp;
+	    if (d != NULL) { /* To be safe in case not key/value pairs */
+	      dp = &cdr(d);
+	      d = *dp;
+	    }
+	  }
+	  if (d == NULL) {
+	    /* Append the pair */
+	    int slen = fieldnamelen;
+	    d = newstring(dupnstr(fieldname,slen),slen);
+	    *dp = d;
+	    dp = &cdr(d);
+	    d = NULL;
+	  }
+	} else {
+	  while(fieldidx-- > 0 && d != NULL) {
+	    dp = &cdr(d);
+	    d = *dp;
+	  }
+	}
+
+	*dp = data;		/* Replace the element */
+	if (d) {
+	  cdr(data) = cdr(d);
+	  cdr(d) = NULL;	/* Disconnect and discard old data */
+	  /* s_free_tree(d); -- GC cleans it out latter .. */
+	}
+
+	UNGCPRO4;
+
+	stickymem = omem;
+	return NULL; /* Be quiet, don't force the caller
+			to store the result into heap just
+		        for latter discard... */
+}
+
+
+
 static conscell *
 sh_get(avl, il)
 	conscell *avl, *il;
 {
-	conscell *plist, *key, *d, *tmp;
+	conscell *plist, *key, *d = NULL;
+	GCVARS3;
 
 	if ((plist = cdar(avl)) == NULL
 	    || (key = cddar(avl)) == NULL
@@ -310,13 +401,15 @@ sh_get(avl, il)
 		d = v_find((const char *)plist->string);
 		if (d == NULL) {
 			/* (setq plist '(key nil)) */
-			d = conststring(key->string);
+			GCPRO3(avl,plist,d);
+			if (ISCONST(key))
+			  d = conststring(key->string, key->slen);
+			else
+			  d = copycell(key);
 			cdr(d) = NIL;
 			d = ncons(d);
 			assign(plist, d, (struct osCmd *)NULL);
-			/* setf preparation */
-			cdar(d)->prev = cadr(v_find((const char*)plist->string));
-			cdar(d)->pflags = 1;
+			UNGCPRO3;
 			return cdar(d);
 		}
 		plist = cdr(d);
@@ -331,9 +424,8 @@ sh_get(avl, il)
 			printf("comparing '%s' and '%s'\n",
 				d->string, key->string);	*/
 		if (STRING(d) && strcmp(d->string, key->string) == 0) {
-			d = copycell(cdr(d));
+			d = copycell(cdr(d)); /* This input is plist elt */
 			cdr(d) = NULL;
-			d->pflags |= 04;
 			return d;
 		}
 		d = cdr(d);
@@ -352,14 +444,8 @@ sh_get(avl, il)
 		cddr(plist) = d;
 
 		stickymem = MEM_MALLOC;
-		cdr(plist) = s_copy_tree(cdr(plist));
-		s_set_prev(plist, cdr(plist));
-		cdr(plist)->pflags = 1;
+		cdr(plist) = s_copy_chain(cdr(plist)); /* input gc-protected */
 		stickymem = oval;
-
-		/* setf preparation */
-		d->prev = cdr(plist);
-		d->pflags = 01 | 04;
 	}
 
 	return d;
@@ -370,7 +456,6 @@ sh_length(avl, il)
 	conscell *avl, *il;
 {
 	char buf[10];
-	conscell *tmp;
 	int len = 0;
 
 	if ((il = cdar(avl)) && LIST(il)) {
@@ -378,8 +463,9 @@ sh_length(avl, il)
 			++len;
 	}
 	sprintf(buf, "%d", len);
+	len = strlen(buf);
 
-	avl = newstring(strsave(buf));
+	avl = newstring(dupnstr(buf,len),len);
 	return avl;
 }
 
@@ -476,12 +562,19 @@ sh_echo(argc, argv)
 	int n;
 
 	--argc, ++argv;
+	/* '-n' ?? */
 	if (argc > 0
 	    && argv[0][0] == '-' && argv[0][1] == 'n' && argv[0][2] == '\0') {
 		--argc, ++argv;
 		n = 0;
 	} else
 		n = 1;
+	/* '--' ?? */
+	if (argc > 0
+	    && argv[0][0] == '-' && argv[0][1] == '-' && argv[0][2] == '\0') {
+		--argc, ++argv;
+	}
+
 	while (--argc >= 0) {
 		fputs(*argv++, stdout);
 		if (argc > 0)
@@ -570,7 +663,7 @@ sh_read(argc, argv)
 	static char *buf = NULL;
 	static u_int bufsize;
 	char *cp, *value = NULL, *bp;
-	int flag, offset;
+	int flag, offset, eoinp = 0;
 
 	if (argc == 1) {
 		fprintf(stderr, USAGE_READ, argv[0]);
@@ -586,7 +679,11 @@ sh_read(argc, argv)
 	flag = 0;
 	bp = NULL;
 	buf[0] = '\0';
-	while (argc > 0 && fgets(buf, bufsize, stdin) != NULL) {
+	while (argc > 0) {
+		if (fgets(buf, bufsize, stdin) == NULL) {
+		  eoinp = 1;
+		  break;
+		}
 		for (cp = buf; argc > 0 && *cp != '\0'; ) {
 			while (*cp != '\0' && WHITESPACE((unsigned)*cp))
 				++cp;
@@ -599,7 +696,7 @@ sh_read(argc, argv)
 					cp = buf + offset;
 					continue;
 				} else
-					goto eoinput;
+					break;
 			}
 			if (!flag) {
 				bp = cp;
@@ -650,11 +747,10 @@ sh_read(argc, argv)
 			v_set(*argv++, value), --argc;
 		}
 	}
-	if (buf[0] == '\0')
-		return 1;
+	/* if (buf[0] == '\0') return 1; */
+	if (eoinp) return 1;
 eoinput:
-	while (argc-- > 0)
-		v_set(*argv++, "");
+	while (argc-- > 0) v_set(*argv++, "");
 	return 0;
 }
 
@@ -808,8 +904,8 @@ sh_export(argc, argv)
 		if (LIST(elist) || elist->string == NULL)
 			continue;
 		for (glist = car(scope); glist != NULL; glist = cddr(glist)) {
-			if (STRING(glist) &&
-			    strcmp(glist->string, elist->string) == 0)
+			if (STRING(glist) && glist->slen == elist->slen &&
+			    memcmp(glist->string, elist->string, elist->slen) == 0)
 				break;
 		}
 		if (glist == NULL)
@@ -920,7 +1016,7 @@ sh_set(argc, argv)
 	memtypes oval;
 	const char *cp;
 	char **kk, *ep;
-	conscell *d, *pd = NULL, *scope, *tmp;
+	conscell *d, *pd = NULL, *scope;
 	char ebuf[32];
 
 	if (argc == 1) {
@@ -934,7 +1030,7 @@ sh_set(argc, argv)
 #ifdef	USE_ALLOCA
 		kk = (char **)alloca((n+1) * sizeof (char *));
 #else
-		kk = (char **)tmalloc((n+1) * sizeof (char *));
+		kk = (char **)emalloc((n+1) * sizeof (char *));
 #endif
 		kk[0] = NULL;
 		/* iterate through all variables */
@@ -954,7 +1050,9 @@ sh_set(argc, argv)
 					putchar('\n');
 				}
 			}
-		/* free(kk); -- tmalloc() on temp memory makes this unnecessary */
+#ifndef USE_ALLOCA
+		free(kk);
+#endif
 		return 0;
 	}
 	--argc, ++argv;
@@ -1001,9 +1099,10 @@ sh_set(argc, argv)
 	stickymem = MEM_MALLOC;
 	if (globalcaller != NULL && globalcaller->argv != NULL)
 		pd = car(globalcaller->argv);
-	/* XX: possible memory leak here! */
 	while (argc-- > 0) {
-		d = newstring(strsave(*argv));
+		int slen = strlen(*argv);
+		d = newstring(dupnstr(*argv,slen),slen);
+		/* These go into  globalcaller->  protected storage */
 		if (pd != NULL)
 			cdr(pd) = d;
 		++argv;
@@ -1052,7 +1151,7 @@ sh_unset(argc, argv)
 				else
 					cddr(pd) = next;
 				cddr(d) = NULL;
-				s_free_tree(d);
+				/* s_free_tree(d); -- GC does it.. */ 
 				/* no point doing anything else in this scope */
 				break;
 			}
