@@ -2,21 +2,35 @@
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
  *
- *	Copyright 1996-1999 Matti Aarnio
+ *	Copyright 1996-2000 Matti Aarnio
  */
 
 /* LINTLIBRARY */
 
 #include "mailer.h"
-#ifdef	HAVE_DB_H
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
+
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
 #if defined(HAVE_DB_185_H) && !defined(HAVE_DB_OPEN2)
 # include <db_185.h>
 #else
+#ifdef HAVE_DB2_DB_H
+# include <db2/db.h>
+#else
+#ifdef HAVE_DB_H
 # include <db.h>
+#else
+#ifdef HAVE_DB1_DB_H
+# include <db1/db.h>
 #endif
+#endif
+#endif
+#endif
+#endif
+
 #include <sys/file.h>
 #include "search.h"
 #include "io.h"
@@ -138,7 +152,7 @@ reopen:
 
 	key.data = (void*)sip->key;
 	key.size = strlen(sip->key) + 1;
-#ifdef HAVE_DB_OPEN2
+#ifdef DB_INIT_TXN
 	rc = (db->get)(db, NULL, &key, &val, 0);
 #else
 	rc = (db->get)(db, &key, &val, 0);
@@ -185,7 +199,7 @@ add_bhash(sip, value)
 
 	val.data = (void*)value;
 	val.size = strlen(value)+1;
-#ifdef HAVE_DB_OPEN2
+#ifdef DB_INIT_TXN
 	rc = (db->put)(db, NULL, &key, &val, 0);
 	/* Emulate BSD DB 1.85 behaviour */
 	if (rc != 0)
@@ -223,7 +237,7 @@ remove_bhash(sip)
 
 	key.data = (void*)sip->key;
 	key.size = strlen(sip->key) + 1;
-#ifdef HAVE_DB_OPEN2
+#ifdef DB_INIT_TXN
 	rc = (db->del)(db, NULL, &key, 0);
 #else
 	rc = (db->del)(db, &key, 0);
