@@ -48,6 +48,9 @@
 
 #if	defined(HAVE_SOCKET) && (defined(HAVE_RESOLVER) || defined(HAVE_YP))
 #include <netdb.h>
+#ifndef EAI_AGAIN
+# include "netdb6.h"
+#endif
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -94,6 +97,10 @@ extern char *optarg;
 extern int   optind;
 char	path[MAXPATHLEN];
 
+#define  ISDIGIT(cc) ('0' <= cc && cc <= '9')
+#define  ISSPACE(cc) (cc == ' ' || cc == '\t')
+
+
 char *host = NULL;
 
 int
@@ -109,17 +116,21 @@ main(argc, argv)
 	struct stat stbuf;
 	int r, pid, dsflag;
 #endif	/* AF_INET */
+	char *expn = NULL;
 
 	progname = argv[0];
 	verbose = debug = errflg = status = user = onlyuser = summary = 0;
 	while (1) {
-	  c = getopt(argc, argv, "dip:r:stu:U:vVSQ");
+	  c = getopt(argc, argv, "dip:r:stu:U:vVSQE:K:");
 	  if (c == EOF)
 	    break;
 	  switch (c) {
 	  case 'd':
 	    ++debug;
 	    break;
+	  case 'E':
+	    expn = optarg;
+	    break;
 	  case 'i':
 	    user = getuid();
 	    onlyuser = 1;
@@ -252,9 +263,18 @@ main(argc, argv)
 #ifdef	AF_INET
 	if (!port || (port && *port != '/')) {
 
-	  struct in_addr naddr;
-	  struct hostent *hp = NULL, he;
-	  struct sockaddr_in sad;
+
+	  typedef union {
+	    struct sockaddr_in  v4;
+#if defined(AF_INET6) && defined(INET6)
+	    struct sockaddr_in6 v6;
+#endif
+	  } Usockaddr;
+
+
+	  struct addrinfo *ai, req;
+	  int rc;
+
 	  struct servent *serv = NULL;
 
 	  int portnum = 174;
@@ -262,7 +282,7 @@ main(argc, argv)
 
 	  if (status < 2 || summary) {
 
-	    if (port && isdigit(*port)) {
+	    if (port && ISDIGIT(*port)) {
 	      portnum = atol(port);
 	    } else if (port == NULL &&
 		       (serv = getservbyname(port ? port : "mailq", "tcp")) == NULL) {
@@ -290,23 +310,52 @@ main(argc, argv)
 		}
 	      }
 	    }
-	    hp = gethostbyname(host);
-	    if (hp == NULL) {
-	      if (inet_pton(AF_INET, host, &naddr.s_addr) == 1) {
-		hp = gethostbyaddr((void*)&naddr, sizeof(naddr), AF_INET);
-		if (hp == NULL){
-		  hp = &he;
-		  he.h_name = host;
-		  he.h_length = 4;
-		}
-	      } else if (hp == NULL) {
-		fprintf(stderr, "%s: cannot find address of %s\n", progname, host);
-		exit(EX_UNAVAILABLE);
+
+	    memset(&req, 0, sizeof(req));
+	    req.ai_socktype = SOCK_STREAM;
+	    req.ai_protocol = IPPROTO_TCP;
+	    req.ai_flags    = AI_CANONNAME;
+	    req.ai_family   = AF_INET;
+	    ai = NULL;
+
+#ifdef HAVE_GETADDRINFO
+	    rc = getaddrinfo(host, "0", &req, &ai);
+#else
+	    rc = _getaddrinfo_(host, "0", &req, &ai,
+			       (debug ? stderr : NULL));
+#endif
+#if defined(INET) && defined(INET6)
+	    {
+	      struct addrinfo *ai6;
+	      req.ai_family   = AF_INET6;
+	      ai6 = NULL;
+	      
+#ifdef HAVE_GETADDRINFO
+	      rc = getaddrinfo(host, "0", &req, &ai6);
+#else
+	      rc = _getaddrinfo_(host, "0", &req, &ai6,
+				 (debug ? stderr : NULL));
+#endif
+	      if (!ai && rc == 0)
+		/* No IPv4, but have IPv6! */
+		ai = ai6;
+	      else if (ai && ai6) {
+		/* Catenate them, FIRST IPv6, then IPv4 things. */
+		struct addrinfo **aip;
+		aip = &ai6->ai_next;
+		while (*aip) aip = &(*aip)->ai_next;
+		*aip = ai;
+		ai = ai6;
 	      }
 	    }
-	    if (strcmp(host, "localhost") != 0 &&
-		strcmp(host, "127.0.0.1") != 0) {
-	      fprintf(stdout,"[%s]\n", hp->h_name);
+#endif
+	    if (! ai) {
+	      fprintf(stderr, "%s: cannot find address of %s\n", progname, host);
+	      exit(EX_UNAVAILABLE);
+	    }
+	    stashmyaddresses(NULL);
+	    if (ai && matchmyaddresses(ai) == 0) {
+	      fprintf(stdout, "[%s]\n", ai->ai_canonname);
 	      nonlocal = 1;
 	    } else
 	      nonlocal = 0;	/* "localhost" is per default a "local" */
@@ -317,60 +366,47 @@ main(argc, argv)
 	    if (status > 1 && !summary)
 	      exit(0);
 	  }
-	  /* try grabbing a port */
-	  fd = socket(PF_INET, SOCK_STREAM, 0);
-	  if (fd < 0) {
-	    fprintf(stderr, "%s: ", progname);
-	    perror("socket");
-	    exit(EX_UNAVAILABLE);
-	  }
-	  sad.sin_family = AF_INET;
-	  if (hp == &he)
-	    sad.sin_addr.s_addr = htonl(naddr.s_addr);
-	  else {
-	    char *addr;
 
-	    eval = EFAULT;
-	    hp_init(hp);
-	    while ((addr = *hp_getaddr()) != NULL) {
-	      sad.sin_addr.s_addr = 0;
-	      sad.sin_port = htons((short)0);
-	      if (bind(fd, (void*)&sad, sizeof sad) < 0) {
-		fprintf(stderr, "%s: ", progname);
-		perror("bind");
-		exit(EX_UNAVAILABLE);
-	      }
-	      sad.sin_port = htons(portnum);
-	      memcpy((void*)&sad.sin_addr, addr, hp->h_length);
-	      if (connect(fd, (void*)&sad, sizeof sad) < 0) {
-		eval = errno;
-		fprintf(stderr, "%s: connect failed to %s", progname,
-			dottedquad(&sad.sin_addr));
-		if ((addr = *hp_nextaddr()) != NULL) {
-		  memcpy((char *)&sad.sin_addr, addr, hp->h_length);
-		  fprintf(stderr, ", trying %s...\n", dottedquad(&sad.sin_addr));
-		} else
-		  putc('\n', stderr);
-		close(fd);
-		if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0){
-		  fprintf(stderr, "%s: ", progname);
-		  perror("socket");
-		  exit(EX_UNAVAILABLE);
-		}
-		continue;
-	      }
-	      break;
-	    }
-	    if (*hp_getaddr() == NULL) {
+	  fd = -1;
+
+	  for (; ai; ai = ai->ai_next) {
+
+	    Usockaddr *sa = (Usockaddr *)ai->ai_addr;
+	    int addrsiz = sizeof(sa->v4);
+
+#if defined(AF_INET6) && defined(INET6)
+	    if (ai->ai_family == AF_INET6) {
+	      addrsiz = sizeof(sa->v6);
+	      sa->v6.sin6_port = htons(portnum);
+	    } else
+#endif
+	      sa->v4.sin_port = htons(portnum);
+
+	    /* try grabbing a port */
+	    fd = socket(ai->ai_family, SOCK_STREAM, 0);
+	    if (fd < 0) {
 	      fprintf(stderr, "%s: ", progname);
-	      errno = eval;
-	      perror("connect");
-	      fprintf(stderr, "%s: unable to contact scheduler on %s\n",
-		      progname, hp->h_name);
-	      exit(EX_UNAVAILABLE);
+	      perror("socket");
+	      continue;
+	    }
+	    while ((rc = connect(fd, (struct sockaddr *)sa, addrsiz)) < 0 &&
+		   (errno == EINTR || errno == EAGAIN));
+
+	    if (rc < 0) {
+	      eval = errno;
+	      close(fd);
+	      fprintf(stderr, "%s: connect failed to %s",
+		      progname, ai->ai_canonname);
+	      fd = -1;
+	      continue;
 	    }
 	  }
-	  docat((char *)NULL, fd);
+	  if (fd >= 0)
+	    docat((char *)NULL, fd);
+	  else {
+	    fprintf(stderr, "%s: connect failed to %s",
+		    progname, host);
+	  }
 	}
 #else	/* !AF_INET */
 	if (strcmp(host, "localhost") == 0 ||
@@ -566,7 +602,7 @@ const char *dirpath;
 	      (dp->d_name[1] == 0 || (dp->d_name[1] == '.' &&
 				      dp->d_name[2] == 0)))
 	    continue; /* . and .. */
-	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0]))
+	  if (ISDIGIT(dp->d_name[0]))
 	    ++n;
 	  else if (strcmp("core",dp->d_name)==0)
 	    sawcore = 1, ++othern;
@@ -660,7 +696,7 @@ checkrouter()
 	  return;
 	}
 	for (dp = readdir(dirp), n = 0; dp != NULL; dp = readdir(dirp)) {
-	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0]))
+	  if (ISDIGIT(dp->d_name[0]))
 	    ++n;
 	}
 #ifdef	BUGGY_CLOSEDIR
@@ -879,7 +915,7 @@ parse(fp)
 	  switch ((int)list) {
 	  case L_CTLFILE:
 	    /* decid:\tfile\tnaddr; off1[,off2,...][\t#message] */
-	    if (!isdigit(buf[0])) {
+	    if (!ISDIGIT(buf[0])) {
 	      if (EQNSTR(buf, names[L_CHANNEL])) {
 		list = L_CHANNEL;
 		break;
@@ -888,18 +924,16 @@ parse(fp)
 		return 1;
 	      }
 	    }
-	    if (!isdigit(buf[0]) ||
+	    if (!ISDIGIT(buf[0]) ||
 		(cp = strchr(buf, ':')) == NULL) {
 	      fprintf(stderr, "%s: %s: orphaned pending recovery\n", progname, buf);
 	      break;
 	    }
 	    *cp++ = '\0';
 	    key = atol(buf);
-	    while (isascii(*cp) && isspace(*cp))
-	      ++cp;
+	    while ( ISSPACE(*cp)) ++cp;
 	    ocp = cp;
-	    while (isascii(*cp) && !isspace(*cp))
-	      ++cp;
+	    while (!ISSPACE(*cp)) ++cp;
 	    *cp++ = '\0';
 if (debug)
   fprintf(stderr," - '%s'\n",ocp);
@@ -939,12 +973,10 @@ if (debug)
 	    memset((void*)v,0,sizeof (struct vertex)+(i-1)*sizeof(long));
 	    v->ngroup = i;
 	    v->cfp = cfp;
-	    while (isascii(*cp) && isspace(*cp))
-	      ++cp;
-	    for (i = 0; isascii(*cp) && isdigit(*cp); ++cp) {
+	    while (ISSPACE(*cp)) ++cp;
+	    for (i = 0; ISDIGIT(*cp); ++cp) {
 	      ocp = cp;
-	      while (isascii(*cp) && isdigit(*cp))
-		++cp;
+	      while (ISDIGIT(*cp)) ++cp;
 	      *cp = '\0';
 	      v->index[i++] = atol(ocp);
 
@@ -1024,10 +1056,10 @@ if (debug)
 	    /* Pick each vertex reference */
 
 	    ++cp;		/* skip the first '>' */
-	    while (*cp && isascii(*cp) && isdigit(*cp)) {
+	    while (ISDIGIT(*cp)) {
 	      int c;
 	      ocp = cp;
-	      while (*cp && isascii(*cp) && isdigit(*cp))
+	      while (ISDIGIT(*cp))
 		++cp;
 	      c = *cp;
 	      *cp = '\0';
@@ -1512,22 +1544,16 @@ printaddrs(v)
 	  cp = v->cfp->contents + v->index[i] + 2;
 	  if (*cp == ' ' || (*cp >= '0' && *cp <= '9'))
 	    cp += _CFTAG_RCPTPIDSIZE;
-	  while (isascii(*cp) && !isspace(*cp))
-	    ++cp;
-	  while (isascii(*cp) && isspace(*cp))
-	    ++cp;
-	  while (isascii(*cp) && !isspace(*cp))
-	    ++cp;
-	  while (isascii(*cp) && isspace(*cp))
-	    ++cp;
+	  while (!ISSPACE(*cp)) ++cp;
+	  while ( ISSPACE(*cp)) ++cp;
+	  while (!ISSPACE(*cp)) ++cp;
+	  while ( ISSPACE(*cp)) ++cp;
 	  ocp = cp;
 	  while (*cp != '\0' && *cp != '\n')
 	    ++cp;
 	  if (*cp-- == '\n') {
-	    while (isascii(*cp) && !isspace(*cp))
-	      --cp;
-	    while (isascii(*cp) && isspace(*cp))
-	      --cp;
+	    while (!ISSPACE(*cp)) --cp;
+	    while ( ISSPACE(*cp)) --cp;
 	    if (cp >= ocp)
 	      *++cp = '\0';
 	    else {
