@@ -76,7 +76,13 @@ static void mq2interpret __((struct mailq *, char *));
 
 static struct mailq *mq2root  = NULL;
 static int           mq2count = 0;
-static int	     mq2max   = 10; /* How many can live simultaneously */
+static int	     mq2max   = 20; /* How many can live simultaneously */
+static int           max_mq_life = 90; /* 90 seconds for an action */
+
+int mq2_active __((void))
+{
+  return (mq2root != NULL);
+}
 
 /* INTERNAL */
 static void mq2_discard(mq)
@@ -295,6 +301,7 @@ static void mq2_read(mq)
 
   if (mq->fd < 0) {
     mq2_discard(mq);
+    return;
     /* Zap! */
   }
 
@@ -343,8 +350,9 @@ static void mq2_read(mq)
 
 
 /* EXTERNAL */
-void mq2_register(fd)
+void mq2_register(fd, addr)
      int fd;
+     Usockaddr *addr;
 {
   struct mailq *mq;
 
@@ -365,6 +373,8 @@ void mq2_register(fd)
   memset(mq, 0, sizeof(*mq));
   
   mq->fd = fd;
+  mq->apopteosis = now + max_mq_life;
+  mq->qaddr = *addr;
 
   mq->nextmailq = mq2root;
   mq2root = mq;
@@ -434,6 +444,11 @@ void mq2_areinsets(rdmaskp, wrmaskp)
       if (mq->fd >= 0 && _Z_FD_ISSET(mq->fd, *wrmaskp)) {
 	mq2_wflush(mq);
       }
+
+      /* Time of forced death ? */
+      if (now > mq->apopteosis)
+	mq2_discard(mq);
+
       mq = mq2;
     }
 
@@ -454,6 +469,15 @@ void mq2_areinsets(rdmaskp, wrmaskp)
  *
  */
 
+int zsfsetfd(fp, fd)
+     Sfio_t *fp;
+     int fd;
+{
+  /* This is *NOT* the SFIO's sfsetfd() -- we do no sfsync() at any point.. */
+  fp->file = fd;
+  return fd;
+}
+
 struct mq2discipline {
   Sfdisc_t D;
   struct mailq *mq;
@@ -477,10 +501,11 @@ ssize_t mq2_sfwrite(sfp, vp, len, discp)
     return i;
 }
 
-static void mq2_thread_report __((struct mailq *mq, int mode));
-static void mq2_thread_report(mq, mode)
+static void mq2_thread_report __((struct mailq *mq, int mode, char *channel, char *host));
+static void mq2_thread_report(mq, mode, channel, host)
      struct mailq *mq;
      int mode;
+     char *channel, *host;
 {
   Sfio_t *fp;
   struct mq2discipline mq2d;
@@ -497,20 +522,29 @@ static void mq2_thread_report(mq, mode)
 
   sfdisc(fp, &mq2d.D);
 
-  thread_report(fp, mode);
 
-  sfsetfd(fp, -1);
+  if (channel && host)
+    thread_detail_report(fp, mode, channel, host);
+  else
+    thread_report(fp, mode);
+
+  zsfsetfd(fp, -1);
   sfclose(fp);
 }
 
 
 /* INTERNAL */
-static int mq2cmd_show(mq,s)
+static int mq2cmd_etrn(mq,s)
      struct mailq *mq;
      char *s;
 {
   char *t = s;
-  long mode;
+  int rc;
+
+  if (!(mq->auth & MQ2MODE_ETRN)) {
+    mq2_puts(mq, "-No ETRN allowed for you.\n");
+    return 0;
+  }
 
   while (*t && (*t != ' ') && (*t != '\t')) ++t;
   if (*t) *t++ = '\000';
@@ -518,31 +552,92 @@ static int mq2cmd_show(mq,s)
 
   /* 's' points to the first arg, 't' points to string after
      separating white-space has been skipped. */
+  rc = turnme(s);
+  if (rc)
+    mq2_puts(mq, "+OK; an ETRN started something.\n");
+  else
+    mq2_puts(mq, "+OK; an ETRN didn't start anything.\n");
 
-  if (strcmp(s,"SNMP") == 0 && mq->auth & MQ2MODE_SNMP) {
+  return 0;
+}
+
+/* INTERNAL */
+static int mq2cmd_show(mq,s)
+     struct mailq *mq;
+     char *s;
+{
+  char *t = s, *u;
+
+  while (*t && (*t != ' ') && (*t != '\t')) ++t;
+  if (*t) *t++ = '\000';
+  while (*t == ' ' || *t == '\t') ++t;
+  u = t;
+  while (*u && (*u != ' ') && (*u != '\t')) ++u;
+  if (*u) *u++ = '\000';
+  while (*u == ' ' || *u == '\t') ++u;
+
+  /* 's' points to the first arg, 't' points to string after
+     separating white-space has been skipped. */
+
+  if (strcmp(s,"SNMP") == 0) {
+    if (! (MQ2MODE_SNMP & mq->auth)) /* If not allowed operation, exit! */
+      return -1;
     mq2_puts(mq, "+OK until LF.LF\n");
-    mq2_thread_report(mq, MQ2MODE_SNMP);
+    mq2_thread_report(mq, MQ2MODE_SNMP, NULL, NULL);
     mq2_puts_(mq, ".\n");
     return 0;
   }
 
-  if (strcmp(s,"QUEUE") != 0)
-    return -1; /* XX: OTHER SHOW CMDS ?? */
+  if (strcmp(s,"QUEUE") == 0) {
 
-  mode = -1;
+    if (strcmp(t,"SHORT") == 0) {
 
-  if (strcmp(t,"SHORT") == 0)
-    mode = MQ2MODE_QQ;
-  else if (strcmp(t,"THREADS") == 0)
-    mode = MQ2MODE_FULL;
+      if (! (MQ2MODE_QQ & mq->auth)) /* If not allowed operation, exit! */
+	return -1;
+      mq2_puts(mq, "+OK until LF.LF\n");
+      mq2_thread_report(mq, MQ2MODE_QQ, NULL, NULL);
+      mq2_puts_(mq, ".\n");
+      return 0;
 
-  if (mode < 0 || ! (mode & mq->auth)) /* If not allowed operation, exit! */
+    } else if (strcmp(t,"THREADS2") == 0) {
+
+      if (! (MQ2MODE_FULL & mq->auth)) /* If not allowed operation, exit! */
+	return -1;
+      mq2_puts(mq, "+OK until LF.LF\n");
+      mq2_thread_report(mq, MQ2MODE_FULL2, NULL, NULL);
+      mq2_puts_(mq, ".\n");
+      return 0;
+
+    } else if (strcmp(t,"THREADS") == 0) {
+
+      if (! (MQ2MODE_FULL & mq->auth)) /* If not allowed operation, exit! */
+	return -1;
+      mq2_puts(mq, "+OK until LF.LF\n");
+      mq2_thread_report(mq, MQ2MODE_FULL, NULL, NULL);
+      mq2_puts_(mq, ".\n");
+      return 0;
+
+    }
+    /* Unknown! */
     return -1;
+  }
+
+  if (strcmp(s,"THREAD") == 0) {
+    /* SHOW THREAD 'channel' 'host' */
+    char *channel = t;
+    char *host    = u;
+
+    if (! (MQ2MODE_FULL & mq->auth)) /* If not allowed operation, exit! */
+      return -1;
+
+    mq2_puts(mq, "+OK until LF.LF\n");
+    mq2_thread_report(mq, MQ2MODE_FULL, channel, host);
+    mq2_puts_(mq, ".\n");
+    return 0;
+  }
+
+  return -1;
 
-  mq2_puts(mq, "+OK until LF.LF\n");
-  mq2_thread_report(mq, mode);
-  mq2_puts_(mq, ".\n");
-  return 0;
 }
 
 
@@ -581,6 +676,10 @@ static void mq2interpret(mq,s)
     if (mq2cmd_show(mq,t) == 0)
       return;
   }
+  if (strcmp(s,"ETRN") == 0) {
+    mq2cmd_etrn(mq,t);
+    return;
+  }
 
   mq2_puts(mq, "-MAILQ2 No such command; VERB='");
   mq2_puts(mq, s);
