@@ -45,12 +45,13 @@ extern int default_full_content; /* at conf.c */
 
 typedef enum { ACTSET_DELIVERED = 0, ACTSET_FAILED = 1,
 	       ACTSET_RELAYED   = 2, ACTSET_DELAYED = 3,
-	       ACTSET_NONE = 4 } ACTSETENUM;
+	       ACTSET_EXPANDED  = 4, ACTSET_NONE = 5 } ACTSETENUM;
 
 struct not {
 	char	    *not;
 	const char  *message;
 	const char  *orcpt;
+	const char  *inrcpt;
 	const char  *notify;
 	int          notifyflgs;
 	char	    *rcpntp;
@@ -375,6 +376,11 @@ writeheader(errfp, eaddr, no_error_reportp, deliveryform, boundary, actionset)
 			 actionset[ACTSET_DELIVERED]);
 		s = ",";
 	      }
+	      if (actionset[ACTSET_EXPANDED]) {
+		sfprintf(errfp,"%sEXPANDED-OK(%d)",s,
+			 actionset[ACTSET_EXPANDED]);
+		s = ",";
+	      }
 	      s = (*s == ',') ? "]" : "";
 	      sfprintf(errfp,"%s\n", s);
 	    } else {
@@ -384,7 +390,7 @@ writeheader(errfp, eaddr, no_error_reportp, deliveryform, boundary, actionset)
 		sfprintf(errfp,"Content-Type: multipart/report; report-type=delivery-status;\n");
 		sfprintf(errfp,"\tboundary=\"%s\"\n\n",boundary);
 		sfprintf(errfp, "This is MULTIPART/REPORT structured message as defined at RFC 1894.\n\n");
-		sfprintf(errfp, "As your email client software vendor, when will they support this\nreport format by showing its formal part in your preferred language.\n\n");
+		sfprintf(errfp, "Ask your email client software vendor, when will they support this\nreport format by showing its formal part in your preferred language.\n\n");
 		sfprintf(errfp, "--%s\n", boundary);
 		sfprintf(errfp, "Content-Type: text/plain\n");
 	      }
@@ -445,7 +451,8 @@ reporterrs(cfpi, delayreports)
 	long format;
 	char boundarystr[400];
 	char spoolid[30];
-	int actionsets[4]; /* 0:DELIVERED, 1:FAILED, 2:RELAYED, 3:DELAYED */
+	int actionsets[5]; /* 0:DELIVERED, 1:FAILED, 2:RELAYED, 3:DELAYED,
+			      4:EXPANDED */
 	ACTSETENUM thisaction;
 
 	if (cfpi->haderror == 0)
@@ -485,7 +492,9 @@ reporterrs(cfpi, delayreports)
 	wroteheader = 0;
 	lastoffset = cfp->offset[cfp->nlines-1];
 	format = 0L;
-	actionsets[0] = actionsets[1] = actionsets[2] = actionsets[3] = 0;
+	actionsets[0] = actionsets[1] = actionsets[2] = 0;
+	actionsets[3] = actionsets[4] = 0;
+
 	for (i = 0; i < cfp->nlines; ++i, ++lp) {
 	  cp = cfp->contents + *lp;
 
@@ -560,10 +569,12 @@ reporterrs(cfpi, delayreports)
 	      ++action;
 	      cp2 = strchr(action,'\001');
 
-	      if (memcmp       (action,"delayed",  7)==0) {
+	      if        (memcmp(action,"delayed",  7)==0) {
 		thisaction = ACTSET_DELAYED;
 	      } else if (memcmp(action,"delivered",9)==0) {
 		thisaction = ACTSET_DELIVERED;
+	      } else if (memcmp(action,"expanded", 8)==0) {
+		thisaction = ACTSET_EXPANDED;
 	      } else if (memcmp(action,"relayed",  7)==0) {
 		thisaction = ACTSET_RELAYED;
 	      } else if (memcmp(action,"failed",   6)==0) {
@@ -592,6 +603,7 @@ reporterrs(cfpi, delayreports)
 	    }
 	    notaries[notarycnt].not        = notary;
 	    notaries[notarycnt].orcpt      = NULL;
+	    notaries[notarycnt].inrcpt     = NULL;
 	    notaries[notarycnt].notify     = NULL;
 	    notaries[notarycnt].message    = NULL;
 	    /* If NOTIFY= is not defined, default is: NOTIFY=FAILURE */
@@ -609,6 +621,13 @@ reporterrs(cfpi, delayreports)
 		  if (*d) *d++ = 0;
 		  continue;
 		}
+		if (CISTREQN(d,"INRCPT=",7)) {
+		  notaries[notarycnt].inrcpt = d+7;
+		  d += 7;
+		  while (*d != 0 && (*d != ' ' && *d != '\t')) ++d;
+		  if (*d) *d++ = 0;
+		  continue;
+		}
 		if (CISTREQN(d,"NOTIFY=",7)) {
 		  char *p;
 		  notaries[notarycnt].notify = d+7;
@@ -630,6 +649,9 @@ reporterrs(cfpi, delayreports)
 		    } else if (CISTREQN(p,"FAILURE",7)) {
 		      notaries[notarycnt].notifyflgs |= NOT_FAILURE;
 		      p += 7;
+		    } else if (CISTREQN(p,"TRACE",5)) {
+		      notaries[notarycnt].notifyflgs |= NOT_TRACE;
+		      p += 5;
 		    } else
 		      break; /* Burp.. Junk! */
 		    if (*p == ',') ++p;
@@ -647,6 +669,7 @@ reporterrs(cfpi, delayreports)
 	    notaries[notarycnt].message = cp;
 
 	    switch (thisaction) {
+	    case ACTSET_EXPANDED:
 	    case ACTSET_DELIVERED:
 	    case ACTSET_RELAYED:
 	      if (notaries[notarycnt].notifyflgs & NOT_SUCCESS)
@@ -692,6 +715,7 @@ reporterrs(cfpi, delayreports)
 	if (!(actionsets[ACTSET_FAILED]   |
 	      actionsets[ACTSET_RELAYED]  |
 	      actionsets[ACTSET_DELAYED]  |
+	      actionsets[ACTSET_EXPANDED] |
 	      actionsets[ACTSET_DELIVERED] )) {
 
 	  /* No reports what so ever ? */
@@ -759,25 +783,40 @@ reporterrs(cfpi, delayreports)
 	  case ACTSET_DELIVERED:
 	    sfprintf(errfp,"DELIVERED (successfully):\n");
 	    break;
+	  case ACTSET_EXPANDED:
+	    sfprintf(errfp,"EXPANDED (to some list or alias):\n");
+	    break;
 	  case ACTSET_NONE:
 	    sfprintf(errfp,"BUG (unknown ACTSET value: %d):\n",
 		     notaries[i].thisaction);
 	    --n;
 	    break;
 	  }
-	  sfprintf(errfp, "  <%s>: ", notaries[i].rcpntp);
+	  if (notaries[i].orcpt) {
+	    sfprintf(errfp, "  Original Recipient:\n    ");
+	    decodeXtext(errfp,notaries[i].orcpt);
+	    sfputc(errfp,'\n');
+	  } else if (notaries[i].inrcpt) {
+	    sfprintf(errfp, "  Arrived Recipient:\n    ");
+	    decodeXtext(errfp, notaries[i].inrcpt);
+	    sfputc(errfp,'\n');
+	  }
+	  sfprintf(errfp, "  Control data:\n");
+	  sfprintf(errfp, "    %s\n", notaries[i].rcpntp);
+	  sfprintf(errfp, "  Diagnostic texts:\n");
 	  ccp = notaries[i].message;
+	  if (*ccp != '\r')
+	    sfprintf(errfp, "    ");
 	  if (strchr(ccp, '\r')) {
-	    sfprintf(errfp, "...\\\n        ");
+	    sfprintf(errfp, "...\\\n    ");
 	    s = ccp;
 	    if (*s == '\r') ++s; /* Skip possible first '\r' */
 	    for (; *s != '\0'; ++s) {
 	      if (*s == '\r') {
-		sfprintf(errfp,"\n        ");
+		sfprintf(errfp,"\n    ");
 	      } else {
 		sfputc(errfp, *s);
 	      }
-	      
 	    }
 	    sfputc(errfp, '\n');
 	  } else
