@@ -41,7 +41,7 @@ extern u_int ranny __((u_int m));
 /* agenda.c */
 extern time_t qipcretry;
 extern int doagenda __((void));
-extern void turnme __((const char *));
+extern int turnme __((const char *));
 
 /* conf.c */
 extern char *qlogdir;
@@ -55,11 +55,12 @@ extern int  sweepinterval;
 extern char *mq2authfile;
 extern char *mailqsock;
 extern void *BADPTR;
-
+extern int   global_report_interval;
 
 /* msgerror.c */
 extern void msgerror __((struct vertex *vp, long offset, const char *message));
-extern void reporterrs __((struct ctlfile *cfpi));
+extern void reporterrs __((struct ctlfile *cfpi, const int delayreport));
+extern void interim_report_run __((void));
 
 /* pipes.c */
 extern int  pipes_create         __((int *tochild, int *fromchild));
@@ -117,20 +118,22 @@ extern struct MIB_MtaEntry *MIBMtaEntry;
 /* threads.c */
 extern struct thread *thread_head, *thread_tail;
 extern void  delete_threadgroup __((struct threadgroup *thgp));
+extern int   delete_thread __((struct thread *));
 extern void  thread_linkin __((struct vertex *cp, struct config_entry *cep, int cfgid, void (*ce_fillin)__((struct threadgroup *, struct config_entry *)) ));
 extern struct web *web_findcreate __((int flag, const char *s));
 extern void        unweb __((int flag, struct web *wp));
-extern int   thread_start __((struct thread *thr, int queue_only_too));
+extern int   thread_start __((struct thread *thr, int queueonly_too));
 extern int   idle_cleanup __((void));
 extern int   pick_next_vertex __((struct procinfo * proc));
 extern int   pick_next_thread __((struct procinfo * proc));
 #ifdef _SFIO_H
 extern void  thread_report __((Sfio_t *, int));
+extern void  thread_detail_report __((Sfio_t *, int, char *, char *));
 #endif
 extern int   idleprocs;
 extern void  web_detangle __((struct vertex *vp, int ok));
 extern void  reschedule __((struct vertex *vp, int factor, int index));
-extern void  thread_reschedule __((struct thread *, time_t, int index));
+extern int   thread_reschedule __((struct thread *, time_t, int index));
 
 /* transport.c */
 extern struct procinfo *cpids;
@@ -170,10 +173,11 @@ extern void report __(( FILE *fpi, FILE *fpo ));
 
 /* mq2.c */
 extern int mq2add_to_mask __((fd_set *rdmaskp, fd_set *wrmaskp, int));
-extern void mq2_register __((int fd));
+extern void mq2_register __((int fd, Usockaddr *));
 extern void mq2_areinsets __((fd_set *rdmaskp, fd_set *wrmaskp));
 extern int  mq2_puts __((struct mailq *, char *s));
 extern int  mq2_putc __((struct mailq *, int c));
+extern int  mq2_active __((void));
 
 /* mq2auth.c */
 extern void mq2auth __((struct mailq *, char *));
