@@ -28,6 +28,7 @@
 #include <sys/resource.h>
 #endif
 #ifdef HAVE_SYS_UN_H
+#include <sys/socket.h>
 #include <sys/un.h>
 #endif
 
@@ -717,7 +718,7 @@ start_child(vhead, chwp, howp)
 	  for (i = 0; ev[i] != NULL; ++i)
 	    sfprintf(sfstdout," %s", ev[i]);
 	  sfprintf(sfstdout," }");
-	  for (i = 0; ev[i] != NULL; ++i)
+	  for (i = 0; av[i] != NULL; ++i)
 	    sfprintf(sfstdout," %s", av[i]);
 	  sfprintf(sfstdout,"\n");
 	}
@@ -1370,7 +1371,7 @@ queryipcinit()
 	  ua.v4.sin_family      = AF_INET;
 	  ua.v4.sin_addr.s_addr = htonl(INADDR_ANY);
 	  querysocket = -1;
-#ifdef INET6_not_now
+#ifdef INET6
 	  querysocket = socket(PF_INET6, SOCK_STREAM, 0);
 #endif
 	  if (querysocket < 0)
