@@ -4,7 +4,7 @@
  */
 /*
  *    Several extensive changes by Matti Aarnio <mea@nic.funet.fi>
- *      Copyright 1991-1999.
+ *      Copyright 1991-2000.
  */
 
 /*
@@ -128,9 +128,27 @@ static void cfparam(str)
 	  }
 	  bindaddr.v4.sin_family = AF_INET;
 	} else {
-	  /* XXX: TODO: Try to see if this is an interface name, and pick
-	     IPv4 and/or IPv6 addresses for that interface. */
-	  bindaddr_set = 0;
+	  if (CISTREQN(param1,"iface:",6)) {
+#if defined(AF_INET6) && defined(INET6)
+	    bindaddr.v6.sin6_family = AF_INET6;
+	    if (zgetifaddress(AF_INET6, param1+6,
+			      (struct sockaddr *)&bindaddr.v6.sin6_addr))
+	      /* Didn't get IPv6 interface address of given name.. */
+#endif
+	      {
+		if (zgetifaddress(AF_INET, param1+6,
+				  (struct sockaddr *)&bindaddr.v4.sin_addr)) {
+		  /* No recognized interface! */
+		  bindaddr_set = 0;
+		} else
+		  /* Got IPv4 type interface address */
+		  bindaddr.v4.sin_family = AF_INET;
+	      }
+	  } else {
+	    /* XXX: TODO: Try to see if this is an interface name, and pick
+	       IPv4 and/or IPv6 addresses for that interface. */
+	    bindaddr_set = 0;
+	  }
 	}
     }
 
@@ -178,6 +196,13 @@ static void cfparam(str)
 	sscanf(param2,"%d",&tarpit_exponent);
     }
 
+    else if (cistrcmp(name, "deliverby") == 0) {
+      if (param1)
+	deliverby_ok = atol(param1);
+      else
+	deliverby_ok = 0;
+    }
+
     /* Two parameter policydb option: DBTYPE and DBPATH */
 
     else if (cistrcmp(name, "policydb") == 0 && param2 /* 2 params */) {
