@@ -1,6 +1,6 @@
 /*
  *  relaytest.c -- module for ZMailer's smtpserver
- *  By Matti Aarnio <mea@nic.funet.fi> 1997-1999
+ *  By Matti Aarnio <mea@nic.funet.fi> 1997-2000
  *
  */
 
@@ -18,13 +18,25 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <errno.h>
-#ifdef HAVE_DB_H
+
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
 #if defined(HAVE_DB_185_H) && !defined(HAVE_DB_OPEN2)
 # include <db_185.h>
 #else
+#ifdef HAVE_DB2_DB_H
+# include <db2/db.h>
+#else
+#ifdef HAVE_DB_H
 # include <db.h>
+#else
+#ifdef HAVE_DB1_DB_H
+# include <db1/db.h>
 #endif
 #endif
+#endif
+#endif
+#endif
+
 #ifdef HAVE_NDBM_H
 #define datum Ndatum
 #include <ndbm.h>
@@ -219,7 +231,7 @@ int *rlenp;			/* result length ptr ! */
 #ifdef HAVE_GDBM_H
     Gdatum Gkey, Gresult;
 #endif
-#ifdef HAVE_DB_H
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
     DBT Bkey, Bresult;
     int rc;
 #endif
@@ -262,7 +274,7 @@ int *rlenp;			/* result length ptr ! */
 	break; /* some compilers complain, some produce bad code
 		  without this... */
 #endif
-#ifdef HAVE_DB_H
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
     case _dbt_btree:
 
 
@@ -272,7 +284,7 @@ int *rlenp;			/* result length ptr ! */
 	Bkey.data = (void *) qptr;
 	Bkey.size = qlen;
 
-#ifdef HAVE_DB_OPEN2
+#ifdef DB_INIT_TXN
 	rc = (rel->btree->get) (rel->btree, NULL, &Bkey, &Bresult, 0);
 #else
 	rc = (rel->btree->get) (rel->btree, &Bkey, &Bresult, 0);
@@ -297,7 +309,7 @@ int *rlenp;			/* result length ptr ! */
 	Bkey.data = (void *) qptr;
 	Bkey.size = qlen;
 
-#ifdef HAVE_DB_OPEN2
+#ifdef DB_INIT_TXN
 	rc = (rel->bhash->get) (rel->bhash, NULL, &Bkey, &Bresult, 0);
 #else
 	rc = (rel->bhash->get) (rel->bhash, &Bkey, &Bresult, 0);
@@ -608,7 +620,7 @@ int whosonrc;
     if (cistrcmp(rel->dbtype, "gdbm") == 0)
 	rel->dbt = _dbt_gdbm;
 #endif
-#ifdef HAVE_DB_H
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
     if (cistrcmp(rel->dbtype, "btree") == 0)
 	rel->dbt = _dbt_btree;
     if (cistrcmp(rel->dbtype, "bhash") == 0)
@@ -644,12 +656,15 @@ int whosonrc;
 	openok = (rel->gdbm != NULL);
 	break;
 #endif
-#ifdef HAVE_DB_H
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
     case _dbt_btree:
 	/* Append '.db' to the name */
 	sprintf(dbname, "%s.db", rel->dbpath);
 #ifdef HAVE_DB_OPEN2
 	rel->btree = NULL;
+#ifndef DB_RDONLY
+# define DB_RDONLY O_RDONLY
+#endif
 	db_open(dbname, DB_BTREE, DB_RDONLY, 0644, NULL, NULL, &rel->btree);
 #else
 	rel->btree = dbopen(dbname, O_RDONLY, 0644, DB_BTREE, NULL);
@@ -662,6 +677,9 @@ int whosonrc;
 	sprintf(dbname, "%s.dbh", rel->dbpath);
 #ifdef HAVE_DB_OPEN2
 	rel->bhash = NULL;
+#ifndef DB_RDONLY
+# define DB_RDONLY O_RDONLY
+#endif
 	db_open(dbname, DB_HASH, DB_RDONLY, 0644, NULL, NULL, &rel->bhash);
 #else
 	rel->bhash = dbopen(rel->dbpath, O_RDONLY, 0644, DB_HASH, NULL);
