@@ -11,6 +11,8 @@
 #define const
 #endif
 
+#include <string.h>
+
 const char *rfc821_error = 0;	/* Error text */
 const char *rfc821_error_ptr = 0;	/* scan position of the error */
 
@@ -46,7 +48,13 @@ static const char *no_input = "No input";
 
 /* Could use 'unsigned short' here, but the Alpha machines dislike
    anything smaller than int (32 bit) */
-static int char_array[] =
+static
+#if defined(__alpha)||defined(__alpha__)
+ int
+#else
+ short
+#endif
+ char_array[] =
 {
 		       /* /------------------------------ CHAR_ALPHA
 			  |  /--------------------------- CHAR_ALNUM
@@ -611,8 +619,8 @@ int strict;
 
     if (*s == 'i' || *s == 'I') {
 	/* Wow! Possibly IPv6 prefix! */
-	if (cistrncmp(s, "IPv6:", 5) == 0 /* ||
-	    cistrncmp(s, "IPv6.", 5) == 0  */  ) {
+	if (strncasecmp(s, "IPv6:", 5) == 0 /* ||
+	    strncasecmp(s, "IPv6.", 5) == 0  */  ) {
 	    p = rfc821_v6dotnum(s + 5, strict);
 	    if (p == s + 5)
 		return s;
@@ -721,11 +729,18 @@ int strict;
 	    return p;
 	rfc821_error_ptr = s;
 	rfc821_error = "RFC821 Domain \"#numbers\" has inadequate count of numbers";
-	return s;		/* Failure, don't advance */
+	return s;	/* Failure, don't advance */
     }
-    allnum = 1;			/* Collect info about all fields being numeric..
-				   To accept  "1302.watstar.waterloo.edu" et.al.
-				   but not something which looks like all numbers.. */
+    allnum = 1;		/* Collect info about all fields being numeric..
+			   To accept  "1302.watstar.waterloo.edu" et.al.
+			   but not something which looks like all numbers.. */
+
+    if (*p == '.') {
+      rfc821_error = "A domain-name does not start with a dot (.)";
+      rfc821_error_ptr = p;
+      return s;
+    }
+
     q = rfc821_name(p, strict, &allnum);
 
     while (p && q > p && *q == '.') {
@@ -771,6 +786,11 @@ int strict;
 					   only the <local-part> */
 	    return p;
     }
+    if (*p == ':') {
+	rfc821_error_ptr = p;
+	rfc821_error = "Perhaps this should have been a dot (.) instead of colon (:) ?";
+	return s;
+    }
     if (*p != '@') {
 	rfc821_error_ptr = p;
 	rfc821_error = "Missing \"@\" from mailbox definition";
@@ -820,7 +840,7 @@ int strict;
     }
     if (q == s) {
 	rfc821_error_ptr = s;
-	rfc821_error = "Expected an rfc821_string";
+	rfc821_error = "Had characters unsuitable for an rfc821-string";
     }
     return q;			/* Advanced or not.. */
 }
@@ -848,8 +868,13 @@ int strict;
 	}
 #endif
 	q = rfc821_string(p, strict);
-	if (q == p)
-	    return s;		/* Missing string */
+	if (q == p) {
+	  if (*q == '@') {
+	    rfc821_error = "Localpart must not end with unquoted dot!";
+	  }
+	  return s;		/* Missing string */
+	}
+	
     }
     return q;
 }
@@ -925,6 +950,7 @@ char *s;			/* Stretched RFC821 a bit here.. !%-hacks */
 int strict;
 {
     char *p = s, *q;
+    int _ok = 0;
 
     if (!s || !*s)
 	return s;		/* Pathological termination */
@@ -935,7 +961,8 @@ int strict;
 		return s;	/* Uh... */
 	    if (*q == '%' || *q == '!') {
 		p = q + 1;
-		if (!is_821_alnum(*p) && *p != '"') {
+		_ok = (*q == '!' && *p == '_');
+		if (!is_821_alnum(*p) && !_ok && *p != '"') {
 		  rfc821_error_ptr = q;
 		  if (*q == '%') {
 		    rfc821_error = "After a '%', a non alphanumeric element";
@@ -953,7 +980,8 @@ int strict;
 	    return s;		/* Uh... */
 	if (*q == '%' || *q == '!') {
 	    p = q + 1;
-	    if (!is_821_alnum(*p) && *p != '"') {
+	    _ok = (*q == '!' && *p == '_');
+	    if (!is_821_alnum(*p) && !_ok && *p != '"') {
 	      rfc821_error_ptr = q;
 	      if (*q == '%') {
 		rfc821_error = "After a '%', a non alphanumeric element";
@@ -982,6 +1010,11 @@ int strict;
     }
     if (*p == '@' && (q = rfc821_adl(p, strict)) && (q > p)) {
 	p = q;
+	if (*p == '>' || *p == ' ') {
+	    rfc821_error_ptr = p;
+	    rfc821_error = "No local part before leading @-character ?";
+	    return s;
+	}
 	if (*p != ':') {
 	    rfc821_error_ptr = p;
 	    rfc821_error = "Missing colon (:) from <@xxx:yyy@zzz>";
@@ -1025,17 +1058,17 @@ int strict;
     }
     q = rfc821_path2(p, strict);
     if (q == p) {
-	/* Report whatever part2() reports as an error */
+	/* Report whatever path2() reports as an error */
 	return s;
     }
     if (*q == ' ' || *q == '\t') {
 	/* Sometimes sending systems have:  <foo@foo   >, painfull.. */
-	for (q = q + 1; (*p == ' ' || *p == '\t');)
-	    ++p;
+	p = q;
+	while (*p == ' ' || *p == '\t') ++p;
 	/* Ok, copy down the rest of the input, and thus save the day..
 	   Purist would just report an error, but lets be lenient, when
 	   it is fairly easy, and painless to do.. */
-	if (*p == '>')
+	if (p > q)
 	    strcpy(q, p);
     }
     if (*q != '>') {
