@@ -307,6 +307,11 @@ const char *buf, *cp;
 	if (dsn_ok)
 	  type(SS, -250, NULL, "DSN");		/* RFC 1891 */
 
+	if (deliverby_ok == 0)
+	  type(SS, -250, NULL, "DELIVERBY");	/* RFC 2852 */
+	else if (deliverby_ok > 0)
+	  type(SS, -250, NULL, "DELIVERBY %d", deliverby_ok);
+
 	if (rcptlimitcnt > 100)
 	  type(SS, -250, NULL, "X-RCPTLIMIT %d", rcptlimitcnt);
 
@@ -360,6 +365,9 @@ int insecure;
 
     SS->sender_ok = 0;		/* Set it, when we are sure.. */
 
+    SS->deliverby_time  = 0;		/* RFC 2852 */
+    SS->deliverby_flags = 0;
+
     /* For ESMTP SIZE-option use we need to know
        how much space we have, it is easiest by
        opening the spool file here, and asking
@@ -506,7 +514,7 @@ int insecure;
       addrlen = s - cp;	/* Length */
     }
 
-    /* BODY=8BITMIME SIZE=nnnn ENVID=xxxxxx RET=xxxx */
+    /* BODY=8BITMIME SIZE=nnnn ENVID=xxxxxx RET=xxxx BY=nnnn;FF */
     SS->sizeoptval = -1;
     SS->sizeoptsum = -1;
     drpt_envid = NULL;
@@ -641,6 +649,70 @@ int insecure;
 	    }
 	    continue;
 	}
+	if (deliverby_ok >= 0 && CISTREQN("BY=", s, 3)) {
+	    /* RFC 2852: DELIVERBY extension */
+	    int neg = 0;
+	    int val = 0;
+	    int cnt = 0;
+	    p = s + 3;
+	    if (*p == '-') neg = *p++; /* non-zero flag */
+	    while ('0' <= *p && *p <= '9') {
+	      val = val * 10 + (*p - '0');
+	      ++p; ++cnt;
+	    }
+	    if (cnt > 9) {
+	    invalid_by_data:
+	      smtp_tarpit(SS);
+	      type821err(SS, 501, m554, buf, "Invalid data at BY= parameter: '%.200s'", s);
+	      rc = 1;
+	      break;
+	    }
+	    if (neg) val = -val;
+	    if (*p != ';') goto invalid_by_data;
+	    ++p;
+	    neg = 0;
+	    while (*p && *p != ' ' && *p != '\t') {
+	      switch(*p) {
+	      case 'T': case 't':
+		if (neg & DELIVERBY_T) goto invalid_by_data;
+		neg |= DELIVERBY_T;
+		break;
+	      case 'N': case 'n':
+		if (neg & (DELIVERBY_R|DELIVERBY_N)) goto invalid_by_data;
+		/* N and R are exclusive */
+		neg |= DELIVERBY_N;
+		break;
+	      case 'R': case 'r':
+		if (neg & (DELIVERBY_R|DELIVERBY_N)) goto invalid_by_data;
+		/* R and N are exclusive */
+		neg |= DELIVERBY_R;
+		break;
+	      default:
+		goto invalid_by_data;
+		break;
+	      }
+	      ++p;
+	    }
+	    if ((neg & (DELIVERBY_N|DELIVERBY_R)) == 0)
+	      goto invalid_by_data; /* Neither N or R ?! */
+	    if ((neg & DELIVERBY_R) && val <= 0) {
+	      type(SS, 501, m554,
+		   "The strict delivery deadline is already past: BY=%d;R%s",
+		   val, (neg & DELIVERBY_T) ? "T":"");
+	      rc = 1;
+	      break;
+	    }
+	    if ((neg & DELIVERBY_R) &&
+		deliverby_ok > 0 && val < deliverby_ok) {
+	      type(SS, 553, m571, "Too small short delivery deadline value given: %d\n", val);
+	      rc = 1;
+	      break;
+	    }
+	    SS->deliverby_time  = time(NULL) + val;
+	    SS->deliverby_flags = neg;
+	    s = p;
+	    continue;
+	}
 
 	smtp_tarpit(SS);
 	type(SS, 501, m554, "Unknown MAIL FROM:<> parameter: %s", s);
@@ -883,6 +955,11 @@ int insecure;
     if (ident_flag)
 	fprintf(SS->mfp, "identinfo %s\n", SS->ident_username);
 
+    if (SS->smtpfrom) free((void*)SS->smtpfrom);
+    SS->smtpfrom = malloc(addrlen+1);
+    memcpy((void*)(SS->smtpfrom), cp, addrlen);
+    *((char*)(&SS->smtpfrom[addrlen])) = 0;
+
     if (addrlen == 0)
 	fputs("channel error\n", SS->mfp);	/* Empty  MAIL FROM:<> */
     else {
@@ -959,7 +1036,7 @@ const char *buf, *cp;
     const char *drpt_notify, *drpt_orcpt;
     const char *newcp = NULL;
     const char *srcrtestatus = "";
-    int addrlen = 0, notifylen = 0, orcptlen = 0;
+    int addrlen = 0, notifylen = 0, orcptlen = 0, notifyflgs;
     int strict = STYLE(SS->cfinfo, 'R');
     int sloppy = STYLE(SS->cfinfo, 'S');
 
@@ -1137,6 +1214,7 @@ const char *buf, *cp;
 	return;
     }
     drpt_notify = NULL;
+    notifyflgs  = 0;
     drpt_orcpt = NULL;
 
     while (*s) {
@@ -1146,6 +1224,12 @@ const char *buf, *cp;
 	    if (strict && !sloppy) break;
 	}
 	/* IETF-NOTARY  SMTP-DSN extensions */
+
+#define NOTIFY_SUCCESS 1
+#define NOTIFY_FAILURE 2
+#define NOTIFY_DELAY   4
+#define NOTIFY_NEVER   8
+
 	if (dsn_ok && CISTREQN("NOTIFY=", s, 7)) {
 	    if (drpt_notify) {
 		smtp_tarpit(SS);
@@ -1156,13 +1240,13 @@ const char *buf, *cp;
 	    s += 7;
 	    while (*s) {
 		if (CISTREQN("SUCCESS", s, 7))
-		    s += 7;
+		    s += 7, notifyflgs |= NOTIFY_SUCCESS;
 		else if (CISTREQN("FAILURE", s, 7))
-		    s += 7;
+		    s += 7, notifyflgs |= NOTIFY_FAILURE;
 		else if (CISTREQN("DELAY", s, 5))
-		    s += 5;
+		    s += 5, notifyflgs |= NOTIFY_DELAY;
 		else if (CISTREQN("NEVER", s, 5))
-		    s += 5;
+		    s += 5, notifyflgs |= NOTIFY_NEVER;
 		if (*s != ',')
 		    break;
 		++s;
@@ -1275,7 +1359,7 @@ const char *buf, *cp;
 	      type(SS,-450, m471, "This target address is not our MX service");
 	      type(SS,-450, m471, "client, nor you are connecting from address");
 	      type(SS,-450, m471, "that is allowed to openly use us to relay");
-	      type(SS,-450, m471, "to any arbitary address thru us.");
+	      type(SS,-450, m471, "to any arbitrary address thru us.");
 	      type(SS, 450, m471, "We don't accept this recipient: <%.*s>",
 		   addrlen, cp);
 	    }
@@ -1305,7 +1389,7 @@ const char *buf, *cp;
 	      type(SS,-550, m571, "This target address is not our MX service");
 	      type(SS,-550, m571, "client, nor you are connecting from address");
 	      type(SS,-550, m571, "that is allowed to openly use us to relay");
-	      type(SS,-550, m571, "to any arbitary address thru us.");
+	      type(SS,-550, m571, "to any arbitrary address thru us.");
 	      type(SS, 550, m571, "We don't accept this recipient: <%.*s>",
 		   addrlen, cp);
 	    }
@@ -1354,6 +1438,26 @@ const char *buf, *cp;
     if (drpt_notify) {
 	fputc(' ', SS->mfp);
 	fwrite(drpt_notify, 1, notifylen, SS->mfp);
+	if (!(notifyflgs & NOTIFY_NEVER) /* Not 'NEVER' */ &&
+	    !(notifyflgs & NOTIFY_DELAY) /* Not 'DELAY' */ &&
+	    (SS->deliverby_flags & DELIVERBY_N) /* 'N' mode */)
+	  /* RFC 2852: 4.1.4.2: */
+	  fwrite(",DELAY", 1, 6, SS->mfp);
+	if (SS->deliverby_flags & DELIVERBY_T)
+	  fwrite(",TRACE", 1, 6, SS->mfp);
+    } else {
+	fprintf(SS->mfp, " NOTIFY=FAILURE,DELAY");
+	if (SS->deliverby_flags & DELIVERBY_T)
+	  fwrite(",TRACE", 1, 6, SS->mfp);
+    }
+    if (SS->deliverby_time) {
+      fprintf(SS->mfp, " BY=%ld;", SS->deliverby_time);
+      if (SS->deliverby_flags & DELIVERBY_R)
+	fputc('R', SS->mfp);
+      if (SS->deliverby_flags & DELIVERBY_N)
+	fputc('N', SS->mfp);
+      if (SS->deliverby_flags & DELIVERBY_T)
+	fputc('T', SS->mfp);
     }
     if (drpt_orcpt) {
 	fputc(' ', SS->mfp);
@@ -1371,6 +1475,30 @@ const char *buf, *cp;
 	    ++p;
 	}
     }
+    { /* Our received inbound RCPT value */
+	const char *p = cp;
+	const char *ep = cp + addrlen;
+	fputs(" INRCPT=rfc822;", SS->mfp);
+	while (*p && p < ep) {
+	    char c = (*p) & 0xFF;
+	    if ('!' <= c && c <= '~' && c != '+' && c != '=')
+		fputc(c, SS->mfp);
+	    else
+		fprintf(SS->mfp, "+%02X", c);
+	    ++p;
+	}
+
+	fputs(" INFROM=rfc822;", SS->mfp);
+	p = SS->smtpfrom;
+	while (*p) {
+	    char c = (*p) & 0xFF;
+	    if ('!' <= c && c <= '~' && c != '+' && c != '=')
+		fputc(c, SS->mfp);
+	    else
+		fprintf(SS->mfp, "+%02X", c);
+	    ++p;
+	}
+    }
     fputc('\n', SS->mfp);
 
     /* Normal "RCPT TO:<>" data: */
@@ -1445,7 +1573,7 @@ const char *buf, *cp;
 
     cfi = STYLE(SS->cfinfo, 'v');
 
-#if 0				/* The input string is ARBITARY STRING (shudder..) */
+#if 0				/* The input string is ARBITRARY STRING (shudder..) */
     if (*cp == '<')
 	s = rfc821_path(cp, cfi);	/* with < > */
     else
@@ -1498,7 +1626,7 @@ const char *buf, *cp;
 
     cfi = STYLE(SS->cfinfo, 'e');
     if (cfi != 0) {
-#if 0				/* The input string is an arbitary string! */
+#if 0				/* The input string is an arbitrary string! */
 	if (*cp == '<')
 	    s = rfc821_path(cp, cfi);	/* with < > */
 	else
