@@ -37,11 +37,14 @@
 #include <string.h>
 #include "libz.h"
 
+extern char * zpwmatch __((const char *, const char *, long *));
+
 #ifdef HAVE_SECURITY_PAM_APPL_H
 
 static char *username = NULL;
 static char *pword = NULL;
 
+static void clean_reply __((int, const struct pam_message **, struct pam_response *));
 static void clean_reply(int num_msg, const struct pam_message **msg,
 			struct pam_response *reply)
 {
@@ -55,7 +58,7 @@ static void clean_reply(int num_msg, const struct pam_message **msg,
 	    switch (msg[count]->msg_style) {
             case PAM_PROMPT_ECHO_ON:
             case PAM_PROMPT_ECHO_OFF:
-                bzero(reply[count].resp, strlen(reply[count].resp));
+		memset(reply[count].resp, 0, strlen(reply[count].resp));
                 free(reply[count].resp);
                 break;
             case PAM_ERROR_MSG:
@@ -70,8 +73,16 @@ static void clean_reply(int num_msg, const struct pam_message **msg,
     }
 }
 
-static int pam_cons(int num_msg, const struct pam_message **msg,
-		    struct pam_response **resp, void *appdata_ptr)
+static int
+pam_cons __((int, const struct pam_message **,
+	     struct pam_response **, void *));
+
+static int
+pam_cons(num_msg, msg, resp, appdata_ptr)
+     int num_msg;
+     const struct pam_message **msg;
+     struct pam_response **resp;
+     void *appdata_ptr;
 {
     int count = 0;
     struct pam_response *reply;
@@ -114,8 +125,8 @@ static struct pam_conv pam_c = {
     NULL
 };
 
-int zpwmatch(uname,password,uidp)
-     char *uname, *password;
+char * zpwmatch(uname,password,uidp)
+     const char *uname, *password;
      long *uidp;
 {
     pam_handle_t *ph;
@@ -132,12 +143,12 @@ int zpwmatch(uname,password,uidp)
 	ret = pam_authenticate(ph, 0);
 	/* type(NULL,0,NULL,"pam_authentication() ret=%d",ret); */
     }
-
+#if 0
     if (ret == PAM_SUCCESS) {
 	ret = pam_acct_mgmt(ph, 0);
 	/* type(NULL,0,NULL,"pam_acct_mgmt() ret=%d",ret); */
     }
-
+#endif
     val = ret;
     if ((ret = pam_end(ph, ret)) != PAM_SUCCESS) {
 	/* type(NULL,0,NULL,"pam_end() ret=%d",ret); */
@@ -146,53 +157,74 @@ int zpwmatch(uname,password,uidp)
     runastrusteduser();
 
     if (username) {
-	bzero(username, strlen(username));
+	memset(username, 0, strlen(username));
 	free(username);
 	username = NULL;
     }
     if (pword) {
-	bzero(pword, strlen(pword));
+	memset(pword, 0, strlen(pword));
 	free(pword);
 	pword = NULL;
     }
 
-    return (val == PAM_SUCCESS) ? 1 : 0;
+    return (val == PAM_SUCCESS) ? NULL : "Authentication Failed";
 }
 
 #else
 # ifdef HAVE_SHADOW_H
 
-int zpwmatch(uname, password, uidp)
-     char *uname, *password;
+char * zpwmatch(uname, password, uidp)
+     const char *uname, *password;
      long *uidp;
 {
     struct spwd *spw;
     struct passwd *pw;
-    char *cr;
+    int ok = 0;
     
     runasrootuser();
 
     if (lckpwdf() == -1) {
-	spw = NULL;
-    } else
+	spw = NULL; /* Lock failed.. */
+    } else {
 	spw = getspnam(uname);
-    ulckpwdf(); /* Not much we can do here anyway if it fails */
+	ulckpwdf(); /* Unlock */
+    }
+
+    pw = getpwnam(uname); /* Do this as root user, just in case.. */
 
     runastrusteduser();
 
-    if (!spw) return 0; /* No such user */
-    pw = getpwnam(uname);
-    if (!pw) return 0;  /* No such user! HUH! (had shadow-entry, though!) */
-    cr = crypt(password, spw->sp_pwdp);
-    *uidp = pw->pw_uid;
+    if (pw) { 
 
-    return (strcmp(cr, spw->sp_pwdp) == 0);
+      /* Either the   getpwnam()  returns working password out of
+	 the shadow dataset, or a third-party shadow set must be used
+	 to pick also the encrypted data..  Both fetches are done,
+	 and now we do comparisons presuming the first fetch result
+	 is usable encrypted password ...
+	 (Thanks to Eugene Crosser for report.)
+      */
+
+      char *cr = crypt(password, pw->pw_passwd);
+
+      if (strcmp(cr, pw->pw_passwd) == 0)
+	ok = 1;
+      else if (spw) {
+	/* Ok, perhaps the second one contains usable encrypted password ? */
+	cr = crypt(password, spw->sp_pwdp);
+	if (strcmp(cr, spw->sp_pwdp) == 0)
+	  ok = 1;
+      }
+
+      *uidp = pw->pw_uid;
+    }
+
+    return (ok ? NULL : "Authentication Failed");
 }
 
 # else
 
-int zpwmatch(uname,password,uidp)
-     char *uname, *password;
+char * zpwmatch(uname,password,uidp)
+     const char *uname, *password;
      long *uidp;
 {
     struct passwd *pw;
@@ -204,11 +236,13 @@ int zpwmatch(uname,password,uidp)
 
     runastrusteduser();
 
-    if (!pw) return 0; /* No such user */
-    cr = crypt(password, pw->pw_passwd);
-    *uidp = pw->pw_uid;
+    if (pw) {
+      cr = crypt(password, pw->pw_passwd);
+      *uidp = pw->pw_uid;
+    }
 
-    return (strcmp(cr, pw->pw_passwd) == 0);
+    return ((pw && (strcmp(cr, pw->pw_passwd) == 0))
+	    ? NULL : "Authentication Failed");
 }
 # endif /* HAVE_SHADOW_H */
 #endif
