@@ -21,7 +21,6 @@
  */
 
 #include "hostenv.h"
-#include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
@@ -34,13 +33,15 @@
 #include <unistd.h> /* F_LOCK is there at some systems.. */
 #endif
 #include <string.h>
+
+#include "ta.h"
+
 #include "mail.h"
 #include "zsyslog.h"
 #include "zmsignal.h"
 
-#include "malloc.h"
+#include "zmalloc.h"
 #include "libz.h"
-#include "ta.h"
 #include "splay.h"
 
 
@@ -56,7 +57,7 @@ extern RETSIGTYPE wantout __((int));
 extern int optind;
 extern char *optarg;
 extern void prversion __((const char *));
-extern void process __((struct ctldesc *dp, const char *optmsg));
+extern void process __((struct ctldesc *dp, const char *optmsg, const int));
 extern void deliver __((struct ctldesc *dp, struct rcpt *rp));
 
 extern void hp_init();
@@ -75,10 +76,6 @@ struct conshell *envarlist = NULL;
 extern int stickymem;	/* for strsave() */
 int	D_alloc = 0;
 
-#if defined(HAVE_SOCKET) && defined(HAVE_PROTOCOLS_RWHOD_H)
-static int readrwho __((void));
-#endif
-
 static void sig_alrm __((int));
 static void sig_alrm(sig)
 int sig;
@@ -100,11 +97,9 @@ main(argc, argv)
 	char *s;
 	char *optmsg = NULL;
 	int c, errflg, fd;
+	int silent = 0;
 	char *host = NULL;	/* .. and what is my host ? */
 	int matchhost = 0;
-#if	defined(HAVE_SOCKET)
-	struct biffer *nbp;
-#endif
 	struct ctldesc *dp;
 
 	RETSIGTYPE (*oldsig) __((int));
@@ -132,16 +127,11 @@ main(argc, argv)
 	else
 	  ++progname;
 
-	if (geteuid() != 0 || getuid() != 0) {
-	  fprintf(stderr, "%s: not running as root!\n", progname);
-	  exit(EX_NOPERM);
-	}
-
 	errflg = 0;
 	logfile = NULL;
 	channel = "";
 	while (1) {
-	  c = getopt(argc, (char*const*)argv, "c:Vh:m:l:");
+	  c = getopt(argc, (char*const*)argv, "?c:Vh:m:l:s");
 	  if (c == EOF)
 	    break;
 	  switch (c) {
@@ -162,16 +152,25 @@ main(argc, argv)
 	  case 'm':
 	    optmsg = optarg;
 	    break;
+	  case 's':
+	    silent = 1;
+	    break;
 	  default:
 	    ++errflg;
 	    break;
 	  }
 	}
 	if (errflg || optind != argc) {
-	  fprintf(stderr, "Usage: %s [-V] [-l logfile] [-c channel] [-h host] [-m msgstr]\n",
+	  fprintf(stderr, "Usage: %s [-s] [-V] [-l logfile] [-c channel] [-h host] [-m msgstr]\n",
 		  argv[0]);
 	  exit(EX_USAGE);
 	}
+
+	if (geteuid() != 0 || getuid() != 0) {
+	  fprintf(stderr, "%s: not running as root!\n", progname);
+	  exit(EX_NOPERM);
+	}
+
 	setreuid(0, 0);		/* make us root all over */
 
 
@@ -240,7 +239,7 @@ main(argc, argv)
 	      fcntl(FILENO(verboselog), F_SETFD, 1);
 	    }
 	  }
-	  process(dp,optmsg);
+	  process(dp, optmsg, silent);
 	  ctlclose(dp);
 
 	}
@@ -251,9 +250,10 @@ main(argc, argv)
 
 
 void
-process(dp,optmsg)
+process(dp, optmsg, silent)
 	struct ctldesc *dp;
 	const char *optmsg;
+	const int silent;
 {
 	struct rcpt *rp;
 
@@ -263,8 +263,12 @@ process(dp,optmsg)
 	for (rp = dp->recipients; rp != NULL; rp = rp->next) {
 
 	  notary_setxdelay(0);
-	  notaryreport(rp->addr->user, "failed", "5.7.0 (Administrative deletion command)", optmsg);
-	  diagnostic(rp, EX_UNAVAILABLE, 0, "%s", optmsg);
-
+	  if (silent) {
+	    notaryreport(rp->addr->user, "delivered", "2.0.0 (Silent ok)", "");
+	    diagnostic(rp, EX_OK, 0, "");
+	  } else {
+	    notaryreport(rp->addr->user, "failed", "5.7.0 (Administrative deletion command)", optmsg);
+	    diagnostic(rp, EX_UNAVAILABLE, 0, "%s", optmsg);
+	  }
 	}
 }
