@@ -513,38 +513,81 @@ ctlopen(file, channel, host, exitflagp, selectaddr, saparam, matchrouter, mrpara
 		  if (*s) *s++ = 0;
 		  prevrp->notifyflgs = 0;
 		  while (*p) {
-		    if (cistrncmp("NEVER",p,5)==0) {
+		    if (CISTREQN("NEVER",p,5)) {
 		      p += 5;
 		      prevrp->notifyflgs |= _DSN_NOTIFY_NEVER;
-		    } else if (cistrncmp("DELAY",p,5)==0) {
+		    } else if (CISTREQN("DELAY",p,5)) {
 		      p += 5;
 		      prevrp->notifyflgs |= _DSN_NOTIFY_DELAY;
-		    } else if (cistrncmp("SUCCESS",p,7)==0) {
+		    } else if (CISTREQN("SUCCESS",p,7)) {
 		      p += 7;
 		      prevrp->notifyflgs |= _DSN_NOTIFY_SUCCESS;
-		    } else if (cistrncmp("FAILURE",p,7)==0) {
+		    } else if (CISTREQN("FAILURE",p,7)) {
 		      p += 7;
 		      prevrp->notifyflgs |= _DSN_NOTIFY_FAILURE;
+		    } else if (CISTREQN("TRACE",p,5)) {
+		      p += 5;
+		      prevrp->notifyflgs |= _DSN_NOTIFY_TRACE;
 		    } else
 		      break; /* Burp !? */
 		    if (*p == ',') ++p;
 		  }
 		  continue;
 		}
-		if (cistrncmp("ORCPT=",s,6)==0) {
+		if (CISTREQN("BY=",s,3)) {
+		  long val = 0;
+		  int  neg = 0, cnt = 0;
+		  s += 3;
+		  if (*s == '-') neg = 1, ++s;
+		  while ('0' <= *s && *s <= '9') {
+		    val = val * 10L + (*s - '0');
+		    ++cnt;
+		    ++s;
+		  }
+		  if (neg) val = -val;
+		  prevrp->deliverby = val;
+		  if (*s == ';') ++s;
+		  while (*s && *s != ' ' && *s != '\t') {
+		    switch (*s) {
+		    case 'R': case 'r':
+		      prevrp->deliverbyflgs |= _DELIVERBY_R;
+		      break;
+		    case 'N': case 'n':
+		      prevrp->deliverbyflgs |= _DELIVERBY_N;
+		      break;
+		    case 'T': case 't':
+		      prevrp->deliverbyflgs |= _DELIVERBY_T;
+		      break;
+		    default:
+		      break;
+		    }
+		    ++s;
+		  }
+		  while (*s && *s != ' ' && *s != '\t') ++s;
+		  if (*s) *s++ = 0;
+		  continue;
+		}
+		if (CISTREQN("ORCPT=",s,6)) {
 		  s += 6;
 		  prevrp->orcpt = s;
 		  while (*s && *s != ' ' && *s != '\t') ++s;
 		  if (*s) *s++ = 0;
 		  continue;
 		}
-		if (cistrncmp("INRCPT=",s,7)==0) {
+		if (CISTREQN("INRCPT=",s,7)) {
 		  s += 7;
 		  prevrp->inrcpt = s;
 		  while (*s && *s != ' ' && *s != '\t') ++s;
 		  if (*s) *s++ = 0;
 		  continue;
 		}
+		if (CISTREQN("INFROM=",s,7)) {
+		  s += 7;
+		  /* FIXME: pull INFROM into some sensible use.. */
+		  while (*s && *s != ' ' && *s != '\t') ++s;
+		  if (*s) *s++ = 0;
+		  continue;
+		}
 		/* XX: BOO! Unknown value! */
 		while (*s && *s != ' ' && *s != '\t') ++s;
 	      }
