@@ -90,6 +90,9 @@ const char *okstr;
 
 static char *wtthost = NULL; /* NOTARY wtthost: 'While-Talking-To -host' */
 static char *wttip   = NULL;
+static char *wtttaid = NULL;
+       int   wtttaidpid = -1;
+CONVERTMODE wttcvtmode = _CONVERT_UNKNOWN;
 
 void notary_setwtt(host)
 const char *host;
@@ -101,6 +104,24 @@ const char *host;
 	  wtthost = NULL;
 }
 
+void notary_settaid(progname,pid)
+const char *progname;
+int pid;
+{
+	if (wtttaid) free(wtttaid);
+	if (progname)
+	  wtttaid = strdup(progname);
+	else
+	  wtttaid = NULL;
+	wtttaidpid = pid;
+}
+
+void notary_setcvtmode(mode)
+CONVERTMODE mode;
+{
+	wttcvtmode = mode;
+}
+
 void notary_setwttip(ip)
 const char *ip;
 {
@@ -119,7 +140,6 @@ int xdly;
 }
 
 static char *A1 = NULL, *A2 = NULL, *A3 = NULL, *A4 = NULL;
-static const char *notaryfmt = "%s\001%s\001%s\001%s\001%s";
 
 void
 notaryreport(arg1,arg2,arg3,arg4)
@@ -150,7 +170,8 @@ notaryreport(arg1,arg2,arg3,arg4)
 	if (A3) len += strlen(A3);
 	if (A4) len += strlen(A4);
 	if (wtthost) len += strlen(wtthost);
-	if (wttip) len += strlen(wttip)+5;
+	if (wttip)   len += strlen(wttip)+5;
+	if (wtttaid) len += strlen(wtttaid)+9;
 
 	if (!notarybuf)
 	  notarybuf = (char*) malloc(len);
@@ -162,12 +183,16 @@ notaryreport(arg1,arg2,arg3,arg4)
 	  return;
 	}
 
-	sprintf(notarybuf, notaryfmt,
+	sprintf(notarybuf, "%s\001%s\001%s\001%s\001%s",
 		A1?A1:"", A2?A2:"", A3?A3:"", A4?A4:"",
 		wtthost?wtthost:"");
 	if (wttip) {
 	  sprintf(notarybuf + strlen(notarybuf), " (%s)", wttip);
 	}
+	if (wtttaid) {
+	  sprintf(notarybuf + strlen(notarybuf), "\001%s[%d]",
+		  wtttaid, wtttaidpid);
+	}
 }
 
 
@@ -359,6 +384,7 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 
 	  if (sbuf) {
 	    /* Log the diagnostic string to the file */
+	    int oldflg;
 #ifndef SPRINTF_CHAR
 	    len = 
 #endif
@@ -374,8 +400,14 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 					   writing to the log! */
 
 	    ctlsize = lseek(rp->desc->ctlfd, 0, SEEK_END);
+	    oldflg = fcntl(rp->desc->ctlfd, F_GETFL, 0);
+	    fcntl(rp->desc->ctlfd, F_SETFL, oldflg | O_APPEND);
 
 	    rc2 = write(rp->desc->ctlfd, sbuf, len);
+
+	    fcntl(rp->desc->ctlfd, F_SETFL, oldflg);
+
+
 	    if (rc2 != len || rc2 < 0 || len < 0) {
 	      /* UAARGH! -- write failed, must have disk full! */
 #ifdef HAVE_FTRUNCATE
@@ -414,12 +446,21 @@ diagnostic(rp, rc, timeout, fmt, va_alist) /* (rp, rc, timeout, "fmtstr", remote
 		  statmsg, message);
 	  fflush(stdout);
 
-	  if (!lockaddr(rp->desc->ctlfd, rp->desc->ctlmap,
-			rp->lockoffset, _CFTAG_LOCK, mark,
-			(char*)rp->desc->msgfile, rp->addr->host, getpid())) {
-	    /* something went wrong in unlocking it, concurrency problem? */
+	  switch(rp->status) {
+	  case EX_IOERR:
+	  case EX_TEMPFAIL:
+	    if (rp->notifyflgs & _DSN__TEMPFAIL_NO_UNLOCK)
+	      break;
+	  default:
+	    if (!lockaddr(rp->desc->ctlfd, rp->desc->ctlmap,
+			  rp->lockoffset, _CFTAG_LOCK, mark,
+			  (char*)rp->desc->msgfile, rp->addr->host,
+			  getpid())) {
+	      /* FIXME: something went wrong in unlocking it,
+		 FIXME: concurrency problem? */
+	    }
+	    rp->lockoffset = 0;	/* mark this recipient unlocked */
 	  }
-	  rp->lockoffset = 0;	/* mark this recipient unlocked */
 
 	  tasyslog(rp, xdelay, wtthost, wttip, statmsg, message);
 	}
