@@ -106,6 +106,15 @@ extern time_t time();
 extern char *strchr(), *strrchr();
 #endif
 
+static int zsfsetfd(fp, fd)
+     Sfio_t *fp;
+     int fd;
+{
+  /* This is *NOT* the SFIO's sfsetfd() -- we do no sfsync() at any point.. */
+  fp->file = fd;
+  return fd;
+}
+
 extern int check_7bit_cleanness __((struct ctldesc *dp));
 
 struct maildesc {
@@ -133,6 +142,7 @@ extern int writemimeline __(( struct maildesc *mp, FILE *fp, const char *buf, in
 #define MO_BSMTP		0x01000 /* BSMTP-wrapping -- with HIDDENDOT.. */
 #define MO_BESMTP		0x02000 /* Extended BSMTP -- SIZE+8BITMIME    */
 #define MO_BEDSMTP		0x04000 /* EBSMTP + DSN */
+#define MO_BEBSMTP		0x04000 /* ESMTP + DELIVERBY */
 #define MO_WANTSDATE		0x08000 /* Wants "Date:" -header */
 #define MO_WANTSFROM		0x10000 /* Wants "From:" -header */
 #define MO_BSMTPHELO		0x20000 /* Add HELO/EHLO to the BSMTP */
@@ -606,11 +616,35 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	    fprintf(tafp,"RCPT TO:<%s>",rp->addr->user);
 	    /* if (mp->flags & MO_BESMTP) { } */
 	    if (mp->flags & MO_BEDSMTP) {
-	      if (rp->notify)
-		fprintf(tafp," NOTIFY=%s",rp->notify);
+	      if (rp->notifyflgs) {
+		char *s = "";
+		fprintf(tafp," NOTIFY=");
+		if (rp->notifyflgs & _DSN_NOTIFY_NEVER) {
+		  fprintf(tafp,"NEVER");
+		}
+		if (rp->notifyflgs & _DSN_NOTIFY_SUCCESS) {
+		  fprintf(tafp,"SUCCESS");
+		  s = ",";
+		}
+		if (rp->notifyflgs & _DSN_NOTIFY_FAILURE) {
+		  fprintf(tafp,"%sFAILURE",s);
+		  s = ",";
+		}
+		if (rp->notifyflgs & _DSN_NOTIFY_DELAY) {
+		  fprintf(tafp,"%sDELAY",s);
+		}
+	      }
 	      if (rp->orcpt)
 		fprintf(tafp," ORCPT=%s",rp->orcpt);
 	    }
+	    if (mp->flags & MO_BEBSMTP) {
+	      if (rp->deliverby) {
+		fprintf(tafp," BY=%ld;", rp->deliverby);
+		if (rp->deliverbyflgs & _DELIVERBY_R) fputc('R',tafp);
+		if (rp->deliverbyflgs & _DELIVERBY_N) fputc('N',tafp);
+		if (rp->deliverbyflgs & _DELIVERBY_T) fputc('T',tafp);
+	      }
+	    }
 	    if (mp->flags & MO_CRLF) putc('\r',tafp);
 	    putc('\n',tafp);
 	  }
@@ -887,7 +921,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  Sfio_t *mfp = sfnew(NULL, NULL, 64*1024, mfd, SF_READ|SF_WHOLE);
 	  sfseek(mfp, dp->msgbodyoffset, SEEK_SET);
 
-#define MFPCLOSE sfsetfd(mfp, -1); sfclose(mfp);
+#define MFPCLOSE zsfsetfd(mfp, -1); sfclose(mfp);
 
 	  readalready = 0;
 #else
@@ -1164,7 +1198,8 @@ struct maildesc *
 readsmcf(file, mailer)
 	char *file, *mailer;
 {
-	char *cp, *entry, buf[BUFSIZ];
+	char *entry, buf[BUFSIZ];
+	unsigned char *cp;
 	FILE *fp;
 	int i;
 	static struct maildesc m;
@@ -1191,7 +1226,7 @@ readsmcf(file, mailer)
 		    progname);
 	    exit(EX_OSERR);
 	  }
-	  entry = cp;
+	  entry = (char*)cp;
 	  strcpy(entry, buf);
 	  SKIPWHILE(!isspace, cp);
 	  if (isascii(*cp) && isspace(*cp)) {
@@ -1267,9 +1302,9 @@ readsmcf(file, mailer)
 	  }
 	}
 	SKIPWHILE(isspace, cp);
-	m.command = cp;
+	m.command = (char*) cp;
 	SKIPWHILE(!isspace, cp);
-	if (cp == m.command) {
+	if ((char*)cp == m.command) {
 		fprintf(stderr,"%s: bad entry for %s\n",progname, m.name);
 		return NULL;
 	}
@@ -1289,7 +1324,7 @@ readsmcf(file, mailer)
 	while (isascii(*cp) && !isspace(*cp) && i < MD_ARGVMAX) {
 	  if (*cp == '\0')
 	    break;
-	  m.argv[i++] = cp;
+	  m.argv[i++] = (char*) cp;
 	  SKIPWHILE(!isspace, cp);
 	  if (isascii(*cp)) {
 	    *cp++ = '\0';
