@@ -1,4 +1,12 @@
+/*
+ *  vacation -- originally BSD vacation by Eric Allman,
+ *
+ *  Adapted to ZMailer by Rayan Zachariassen, and further
+ *  modified by Matti Aarnio over years 1988(?) thru 1998.
+ */
+
 #include "hostenv.h"
+
 #include <stdio.h>
 #include <ctype.h>
 #ifdef HAVE_STDLIB_H
@@ -25,17 +33,36 @@
 #ifdef  HAVE_GDBM_H
 #include <gdbm.h>
 #include <fcntl.h>
+#else
+#if defined(HAVE_DB_H)||defined(HAVE_DB1_DB_H)||defined(HAVE_DB2_DB_H)
+#if defined(HAVE_DB_185_H) && !defined(HAVE_DB_OPEN2)
+# include <db_185.h>
+#else
+#ifdef HAVE_DB2_DB_H
+# include <db2/db.h>
+#else
+#ifdef HAVE_DB_H
+# include <db.h>
+#else
+#ifdef HAVE_DB1_DB_H
+# include <db1/db.h>
+#endif
+#endif
+#endif
+#endif
+#else
+:error:error:error "To compile, VACATION needs ndbm.h, gdbm.h, or db.h; none found!"
+#endif
 #endif
 #endif
 /* #include "useful.h"  */
 /* #include "userdbm.h" */
 
 #include "mail.h"
-#include "malloc.h"
+#include "zmalloc.h"
 #include "libz.h"
 #include "libc.h"
 
-extern void   sendmessage __((const char *, const char *));
 extern char * newstr __((const char *));
 extern void   setinterval __((time_t));
 extern void   setreply __((void));
@@ -97,13 +124,19 @@ ALIAS *names = NULL;
 
 #ifdef	HAVE_NDBM_H
 DBM *db;
+#define DBT datum
 #else
 #ifdef HAVE_GDBM_H
 GDBM_FILE db;
+#define DBT datum
+#else /* HAVE_DB_H */
+DB *db;
+/* Natural datum type is: DBT */
+#define dptr  data
+#define dsize size
 #endif /* GDBM */
 #endif	/* NDBM */
 
-#define DBT datum
 
 char from[MAXLINE];
 char *subject_str = NULL;	/* Glob subject from input */
@@ -122,6 +155,10 @@ extern void usrerr __((char *));
 extern void syserr __((char *));
 extern void readheaders __((void));
 extern char *strerror __((int));
+static int recent __((void));
+static int junkmail __((void));
+static int nsearch __((char *name, char *str));
+static void sendmessage __((const char *msgf, const char *myname));
 
 const char *progname;
 
@@ -135,7 +172,7 @@ main(argc, argv)
 	ALIAS *cur;
 	time_t interval;
 	char *msgfile = NULL;
-	int ch, iflag;
+	int ch, iflag, ret;
 
 	progname = argv[0];
 
@@ -143,38 +180,39 @@ main(argc, argv)
 	opterr = iflag = 0;
 	interval = -1;
 	while ((ch = getopt(argc,argv,"a:Iir:t:m:d?")) != EOF) {
-		switch ((char)ch) {
-		  case 'a':	/* alias */
-		        if (!(cur = (ALIAS*) malloc((u_int)sizeof(ALIAS))))
-			  break;
-			cur->name = optarg;
-			cur->next = names;
-			names = cur;
-			break;
-		  case 'I':	/* backwards compatible */
-		  case 'i':	/* initialize the database*/
-			iflag = 1;
-			break;
-		  case 'm':	/* set file to get message from */
-			msgfile = optarg;
-			break;
-		  case 'd':	/* No dbm log of sender */
-			dblog = 0;
-			break;
-		  case 't':
-		  case 'r':
-			if (isdigit(*optarg)) {
-				interval = atol(optarg) * (24*60*60);
-				if (interval < 0)
-					usage();
-			}
-			else
-				interval = INT_MAX;
-			break;
-		  case '?':
-		  default:
-			usage();
-		}
+	  switch ((char)ch) {
+	  case 'a':	/* alias */
+	    cur = (ALIAS*) malloc((u_int)sizeof(ALIAS));
+	    if (!cur)
+	      break;
+	    cur->name = optarg;
+	    cur->next = names;
+	    names = cur;
+	    break;
+	  case 'I':	/* backwards compatible */
+	  case 'i':	/* initialize the database*/
+	    iflag = 1;
+	    break;
+	  case 'm':	/* set file to get message from */
+	    msgfile = optarg;
+	    break;
+	  case 'd':	/* No dbm log of sender */
+	    dblog = 0;
+	    break;
+	  case 't':
+	  case 'r':
+	    if (isdigit(*optarg)) {
+	      interval = atol(optarg) * (24*60*60);
+	      if (interval < 0)
+		usage();
+	    }
+	    else
+	      interval = INT_MAX;
+	    break;
+	  case '?':
+	  default:
+	    usage();
+	  }
 	}
 
 	argc -= optind;
@@ -183,104 +221,116 @@ main(argc, argv)
 	/* verify recipient argument */
 #ifdef ZMAILER
 	if (argc == 0) {
-		p = getenv("USER");
-		if (p == NULL) {
-			usrerr
-			  ("Zmailer error: USER environment variable not set");
-			exit(EX_USAGE+101);
-		}
+	  p = getenv("USER");
+	  if (p == NULL) {
+	    usrerr("Zmailer error: USER environment variable not set");
+	    exit(EX_USAGE+101);
+	  }
 	}
 #endif /* ZMAILER */
 
 	if (argc != 1) {
-		if (!iflag)
-			usage();
-		if (!(pw = getpwuid(getuid()))) {
-			fprintf(stderr,
-				"vacation: no such user uid %u.\n", getuid());
-			exit(EX_NOUSER);
-		}
+	  if (!iflag)
+	    usage();
+	  pw = getpwuid(getuid());
+	  if (!pw) {
+	    fprintf(stderr, "vacation: no such user uid %ld.\n", (long)getuid());
+	    exit(EX_NOUSER);
+	  }
 	} else if (!(pw = getpwnam(*argv))) {
-		fprintf(stderr, "vacation: no such user %s.\n", *argv);
-		exit(EX_NOUSER);
+	  fprintf(stderr, "vacation: no such user %s.\n", *argv);
+	  exit(EX_NOUSER);
 	}
 	if (chdir(pw->pw_dir)) {
-		fprintf(stderr,
-			"vacation: no such directory %s.\n", pw->pw_dir);
-		exit(EX_NOUSER);
+	  fprintf(stderr, "vacation: no such directory %s.\n", pw->pw_dir);
+	  exit(EX_NOUSER);
 	}
 
 	/* verify recipient argument */
 #ifdef ZMAILER
 	if (argc == 0) {
-		p = getenv("USER");
-		if (p == NULL) {
-			usrerr
-			  ("Zmailer error: USER environment variable not set");
-			exit(EX_USAGE+102);
-		}
+	  p = getenv("USER");
+	  if (p == NULL) {
+	    usrerr("Zmailer error: USER environment variable not set");
+	    exit(EX_USAGE+102);
+	  }
 	}
 #endif /* ZMAILER */
 
 #ifdef	HAVE_NDBM_H
 	if (dblog)
-		db = dbm_open(VDB, O_RDWR | (iflag ? O_TRUNC|O_CREAT : 0),
-			      S_IRUSR|S_IWUSR);
+	  db = dbm_open(VDB, O_RDWR | (iflag ? O_TRUNC|O_CREAT : 0),
+			S_IRUSR|S_IWUSR);
 #else	/* !NDBM */
 #ifdef HAVE_GDBM_H
 	if (dblog)
-		db = gdbm_open(VDB ".pag" /* Catenates these strings */, 8192,
-			       iflag ? GDBM_NEWDB : GDBM_WRITER,
-			       S_IRUSR|S_IWUSR, NULL );
+	  db = gdbm_open(VDB ".pag" /* Catenates these strings */, 8192,
+			 iflag ? GDBM_NEWDB : GDBM_WRITER,
+			 S_IRUSR|S_IWUSR, NULL );
+#else
+	db = NULL;
+#ifdef HAVE_DB_OPEN2
+	if (dblog)
+	  db_open(VDB ".db", DB_BTREE, DB_CREATE, S_IRUSR|S_IWUSR,
+		  NULL, NULL, &db);
 #else
 	if (dblog)
-		db = dbinit(VDB); /* XX: old DBM ? */
+	  db = dbopen(VDB ".db", iflag ? (O_RDWR|O_CREAT) : O_RDWR,
+		      S_IRUSR|S_IWUSR, DB_BTREE, NULL);
 #endif
 #endif
+#endif
+
+	ret = EX_OK;
+
 	if (dblog && !db) {
-		fprintf(stderr, "vacation: %s.* database file(s): %s\n", 
-			VDB, strerror(errno));
-		exit(EX_CANTCREAT);
+	  fprintf(stderr, "vacation: %s.* database file(s): %s\n", 
+		  VDB, strerror(errno));
+	  exit(EX_CANTCREAT);
 	}
 
 	if (interval != -1)
-		setinterval(interval);
+	  setinterval(interval);
+
+	if (!iflag) {
+
+	  cur = (ALIAS *)malloc((u_int)sizeof(ALIAS));
+	  if (!cur) {
+	    ret = EX_TEMPFAIL;
+	  } else {
+	    cur->name = pw->pw_name;
+	    cur->next = names;
+	    names = cur;
+
+	    /* read message from standard input (just from line) */
+	    readheaders();
+	    purge_input();
+	    if (!recent()) {
+	      setreply();
+	      sendmessage(msgfile,pw->pw_name);
+	    }
+	  }
+	}
 
-	if (iflag) {
 #ifdef	HAVE_NDBM_H
-		if (dblog)
-			dbm_close(db);
+	if (dblog)
+	  dbm_close(db);
 #else
 #ifdef HAVE_GDBM_H
-		if (dblog)
-			gdbm_close(db);
-#else
-		if (dblog)
-			XX: eh, what ?
-#endif
-#endif
-		exit(EX_OK);
-	}
-
-	if (!(cur = (ALIAS *)malloc((u_int)sizeof(ALIAS)))) {
-		exit(EX_TEMPFAIL);
-	}
-	cur->name = pw->pw_name;
-	cur->next = names;
-	names = cur;
-
-	/* read message from standard input (just from line) */
-	readheaders();
-	purge_input();
-	if (!recent()) {
-		setreply();
-		if (dblog)
-			dbm_close(db);
-		sendmessage(msgfile,pw->pw_name);
-	}
 	if (dblog)
-		dbm_close(db);
-	exit(EX_OK);
+	  gdbm_close(db);
+#else
+#ifdef HAVE_DB_OPEN2
+	if (dblog)
+	  db->close(db, 0);
+#else
+	if (dblog)
+	  db->close(db);
+#endif
+#endif
+#endif
+
+	exit(ret);
 }
 /*
 **  SENDMESSAGE -- send a message to a particular user.
@@ -296,7 +346,7 @@ main(argc, argv)
 **		sends mail to 'user' using /usr/lib/sendmail.
 */
 
-void
+static void
 sendmessage(msgf, myname)
 	const char *msgf;
 	const char *myname;
@@ -471,7 +521,7 @@ readheaders()
 		case 'P':		/* "Precedence:" */
 			cont = 0;
 			if (strncasecmp(buf, "Precedence", 10) ||
-			    buf[10] != ':' && buf[10] != ' ' && buf[10] != '\t')
+			    (buf[10] != ':' && buf[10] != ' ' && buf[10] != '\t'))
 				break;
 			if (!(p = strchr(buf, ':')))
 				break;
@@ -525,7 +575,7 @@ findme:			for (cur = names; !tome && cur; cur = cur->next)
  * nsearch --
  *	do a nice, slow, search of a string for a substring.
  */
-int
+static int
 nsearch(name, str)
 	register char *name, *str;
 {
@@ -541,16 +591,21 @@ nsearch(name, str)
  * junkmail --
  *	read the header and return if automagic/junk/bulk mail
  */
-int
+static int
 junkmail()
 {
 	static struct ignore {
 		const char	*name;
 		int		len;
 	} ignore[] = {
-		"-request", 8,		"postmaster", 10,	"uucp", 4,
-		"mailer-daemon", 13,	"mailer", 6,		"-relay", 6,
-		NULL, 0
+		{ "-request", 8 },
+		{ "postmaster", 10 },
+		{ "uucp", 4 },
+		{ "mailer-daemon", 13 },
+		{ "mailer", 6 },
+		{ "-relay", 6 },
+		{ NULL, 0 }
+		
 	};
 	register struct ignore *cur;
 	register int len;
@@ -605,13 +660,17 @@ purge_input()
  *	find out if user has gotten a vacation message recently.
  *	use memcpy for machines with alignment restrictions
  */
-int recent()
+static int
+recent()
 {
 	DBT key, data;
 	time_t then, next;
 
 	if (!dblog) return 0;
 
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+
 	/* get interval time */
 	key.dptr = VIT;
 	key.dsize = sizeof(VIT);
@@ -621,7 +680,13 @@ int recent()
 #ifdef HAVE_GDBM_H
 	data = gdbm_fetch(db, key);
 #else
-    XX:XX:XX:XX:XX:XX:XX:XX
+#ifdef DB_INIT_TXN
+	if (db->get(db, NULL, &key, &data, 0) != 0)
+	  data.dptr = NULL;
+#else
+	if (db->get(db, &key, &data, 0) != 0)
+	  data.dptr = NULL;
+#endif
 #endif
 #endif
 	if (data.dptr == NULL)
@@ -629,6 +694,9 @@ int recent()
 	else
 		memcpy(&next, data.dptr, sizeof(next));
 
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+
 	/* get record for this address */
 	key.dptr = from;
 	key.dsize = strlen(from);
@@ -638,7 +706,13 @@ int recent()
 #ifdef HAVE_GDBM_H
 	data = gdbm_fetch(db, key);
 #else
-    XX:XX:XX:XX:XX:XX:XX:XX
+#ifdef DB_INIT_TXN
+	if (db->get(db, NULL, &key, &data, 0) != 0)
+	  data.dptr = NULL;
+#else
+	if (db->get(db, &key, &data, 0) != 0)
+	  data.dptr = NULL;
+#endif
 #endif
 #endif
 	if (data.dptr) {
@@ -661,16 +735,26 @@ setinterval(interval)
 
 	if (!dblog) return;
 
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+
 	key.dptr = VIT;
 	key.dsize = sizeof(VIT);
+
 	data.dptr = (void*)&interval;
 	data.dsize = sizeof(interval);
+
 #ifdef HAVE_NDBM_H
 	dbm_store(db, key, data, DBM_REPLACE);
 #else
 #ifdef HAVE_GDBM_H
 	gdbm_store(db, key, data, GDBM_REPLACE);
 #else
+#ifdef DB_INIT_TXN
+	db->put(db, NULL, &key, &data, 0);
+#else
+	db->put(db, &key, &data, 0);
+#endif
 #endif
 #endif
 }
@@ -687,17 +771,27 @@ setreply()
 
 	if (!dblog) return;
 
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+
 	key.dptr = from;
 	key.dsize = strlen(from);
+
 	time(&now);
 	data.dptr = (void*)&now;
 	data.dsize = sizeof(now);
+
 #ifdef HAVE_NDBM_H
 	dbm_store(db, key, data, DBM_REPLACE);
 #else
 #ifdef HAVE_GDBM_H
 	gdbm_store(db, key, data, GDBM_REPLACE);
 #else
+#ifdef DB_INIT_TXN
+	db->put(db, NULL, &key, &data, 0);
+#else
+	db->put(db, &key, &data, 0);
+#endif
 #endif
 #endif
 }
@@ -705,6 +799,6 @@ setreply()
 void
 usage()
 {
-	fprintf(stderr,"vacation: [-i] [-d] [-a alias] [-m msgfile] [-r interval] login\n");
+	fprintf(stderr,"vacation: [-i] [-d] [-a alias] [-m msgfile] [-r interval] {login | 'start' | 'stop' }\n");
 	exit(EX_USAGE);
 }
